'''
Definition for the class Parameters. Used for storing the fitting parameters
in GenX.
Programmer: Matts Bjorck
'''

import numpy as np
import string
from genx.gui_logging import iprint

# ==============================================================================


class Parameters:
    """
    Class for storing the fitting parameters in GenX
    """
    # Parameters used for saving the object state
    export_parameters={'data_labels': list, 'init_data': list, 'data': list}

    def __init__(self, model=None):
        self.data_labels=['Parameter', 'Value', 'Fit', 'Min', 'Max', 'Error']
        self.init_data=['', 0.0, False, 0.0, 0.0, 'None']
        self.data=[self.init_data[:]]
        self.model=model
        self.string_dtype="S100"

    def write_h5group(self, group):
        """ Export the members in the object to a h5py group.
        :param group: h5py Group to export to
        :return:
        """
        group['data_labels']=[label.encode('utf-8') for label in self.data_labels]
        # print np.array([r[0] for r in self.data], dtype='S50')
        group['data col 0']=[r[0].encode('utf-8') for r in self.data]
        group['data col 1']=[r[1] for r in self.data]
        group['data col 2']=[r[2] for r in self.data]
        group['data col 3']=[r[3] for r in self.data]
        group['data col 4']=[r[4] for r in self.data]
        group['data col 5']=[r[5].encode('utf-8') for r in self.data]

    def read_h5group(self, group):
        """ Import data to the object from a h5py group

        :param group: h5py Group to import from
        :return:
        """
        self.data_labels=[item.decode('utf-8') for item in list(group['data_labels'][()])]
        self.data=[[c0.decode('utf-8'), float(c1), bool(c2), float(c3), float(c4), c5.decode('utf-8')]
                   for (c0, c1, c2, c3, c4, c5) in
                   zip(group['data col 0'][()], group['data col 1'][()],
                       group['data col 2'][()], group['data col 3'][()],
                       group['data col 4'][()], group['data col 5'][()])]

    def to_dict(self):
        """Creates a dict from the names in the columns, returns a Dict"""
        d={}
        for i in range(len(self.data_labels)):
            d[self.data_labels[i]]=[r[i] for r in self.data]
        return d

    def set_value(self, row, col, value):
        """ Set a value in the parameter grid """
        self.data[row][col]=value

    def get_value(self, row, col):
        """ Get the value in the grid """
        return self.data[row][col]

    def name_in_grid(self, name):
        """ Checks if name is a parameter in the grid

        :param name: string representation of a parameter
        :return:
        """
        par_names=[row[0] for row in self.data]
        return name in par_names

    def get_names(self):
        """ Returns the parameter names

        :return:
        """
        par_names=[row[0] for row in self.data]

        return par_names

    def get_value_by_name(self, name):
        """Get the value for parameter name. Returns None if name can not be found.

        :param name:
        :return: Value or None
        """
        par_names=[row[0] for row in self.data]
        if name in par_names:
            value=self.data[par_names.index(name)][1]
        else:
            value=None
        return value

    def get_fit_state_by_name(self, name):
        """Get the fitting state for parameter name. Returns None if name can not be found.

        :param name:
        :return: int 0 - not found, 1 - fitted, 2 - defined but constant
        """
        par_names=[row[0] for row in self.data]
        state=0
        if name in par_names:
            if self.data[par_names.index(name)][2]:
                state=1
            else:
                state=2
        return state

    def set_fit_state_by_name(self, name, value, state, min_val, max_val):
        """ Set the fit state by name accoring to the following states
        0 -- remove from grid.
        1 -- add if not exist, set parameter to be fitted (True)
        2 -- add if not exist, set parameter to not be fitted (False)

        :param name: parameter name
        :param state: state, see above.
        :return:
        """
        par_names=[row[0] for row in self.data]
        if state==0 and name in par_names:
            self.delete_rows([par_names.index(name), ])
        elif state==1 or state==2:
            if state==1:
                fit=True
            else:
                fit=False
            if name in par_names:
                self.data[par_names.index(name)][1]=value
                self.data[par_names.index(name)][2]=fit
            else:
                self.append()
                self.data[-1][0]=name
                self.data[-1][1]=value
                self.data[-1][2]=fit
                self.data[-1][3]=min_val
                self.data[-1][4]=max_val
                self.data[-1][5]='-'

    def get_len_rows(self):
        return len(self.data)

    def get_len_cols(self):
        return len(self.data[0])

    def get_len_fit_pars(self):
        return sum([row[2] for row in self.data])

    def get_col_headers(self):
        return self.data_labels[:]

    def delete_rows(self, rows):
        ''' Delete the rows in the list rows ...'''
        delete_count=0
        rows=rows[:]
        rows.sort()

        for i in rows:
            # Note index changes as we delete values. thats why rows has to be sorted
            try:
                self.data.pop(i-delete_count)
            except:
                pass
            else:
                delete_count+=1

        return delete_count

    def insert_row(self, row):
        ''' Insert a new row at row(int). '''
        self.data.insert(row, self.init_data[:])

    def move_row_up(self, row):
        """ Move row up.

        :param row: Move row up one row.
        :return: Boolean True if the row was moved, otherwise False.
        """

        if row!=0 and row<self.get_len_rows():
            self.data.insert(row-1, self.data.pop(row))
            return True
        else:
            return False

    def move_row_down(self, row):
        """ Move row up.

        :param row: Move row down one row.
        :return: Boolean True if the row was moved, otherwise False.
        """

        if row<self.get_len_rows()-1:
            self.data.insert(row+1, self.data.pop(row))
            return True
        else:
            return False

    def _sort_key_func(self, item):
        class_name=''
        pname=item[0]
        obj_name=item[0]
        if self.model is not None:
            if item[0].count('.')>0 and self.model.compiled:
                pieces=item[0].split('.')
                obj_name=pieces[0]
                pname=pieces[1]
                obj=self.model.eval_in_model(obj_name)
                class_name=obj.__class__.__name__

        return string.lower(class_name), string.lower(pname), string.lower(obj_name)

    def sort_rows(self):
        """ Sort the rows in the table

        :return: Boolean to indicate success (True)
        """
        self.data.sort(key=self._sort_key_func)
        return True

    def append(self, parameter=None):
        data=list(self.init_data)
        out=ConnectedParameter(self, data)
        if parameter is not None:
            out.name=parameter
        self.data.append(data)
        return out

    def get_fit_pars(self):
        ''' Returns the variables needed for fitting '''
        # print 'Data in the parameters class: ', self.data
        rows=list(range(len(self.data)))
        row_nmb=[nmb for nmb in rows if self.data[nmb][2] and \
                 not self.data[nmb][0]=='']
        funcs=[row[0] for row in self.data if row[2] and not row[0]=='']
        mytest=[row[1] for row in self.data if row[2] and not row[0]=='']
        min=[row[3] for row in self.data if row[2] and not row[0]=='']
        max=[row[4] for row in self.data if row[2] and not row[0]=='']
        return row_nmb, funcs, mytest, min, max

    def get_pos_from_row(self, row):
        '''get_pos_from_row(self) --> pos [int]

        Transform the row row to the position in the fit_pars list
        '''
        rows=list(range(row+1))
        row_nmb=[nmb for nmb in rows if self.data[nmb][2] and \
                 not self.data[nmb][0]=='']
        return len(row_nmb)-1

    def get_sim_pars(self):
        ''' Returns the variables needed for simulation '''
        funcs=[row[0] for row in self.data if not row[0]=='']
        mytest=[row[1] for row in self.data if not row[0]=='']
        return funcs, mytest

    def get_sim_pos_from_row(self, row):
        '''Transform a row to a psoitions in the sim list
        that is returned by get_sim_pars
        '''
        rows=list(range(row+1))
        row_nmb=[nmb for nmb in rows if not self.data[nmb][0]=='']
        return len(row_nmb)-1

    def set_value_pars(self, value):
        ''' Set the values of the parameters '''
        valueindex=0
        for row in self.data:
            if row[2] and not row[0]=='':
                row[1]=value[valueindex]
                valueindex=valueindex+1

    def set_error_pars(self, value):
        ''' Set the errors on the parameters '''
        valueindex=0
        for row in self.data:
            if row[2] and not row[0]=='':
                row[5]=value[valueindex]
                valueindex=valueindex+1

    def clear_error_pars(self):
        ''' clears the errors in the parameters'''
        for row in self.data:
            row[5]='-'

    def set_data(self, data):
        rowi=0
        coli=0
        for row in data:
            for col in row:
                self.set_value(rowi, coli, col)
                coli=coli+1
            rowi=rowi+1
            coli=0

    def get_data(self):
        return self.data[:]

    def get_ascii_output(self):
        '''get_ascii_output(self) --> text [string]

        Returns the parameters grid as an ascii string.
        '''
        text='#'
        # Show the data labels but with a preceeding # to denote a comment
        for label in self.data_labels:
            text+=label+'\t'
        text+='\n'
        for row in self.data:
            for item in row:
                # special handling of floats to reduce the
                # col size use 5 significant digits
                if type(item)==type(10.0):
                    text+='%.4e\t'%item
                else:
                    text+=item.__str__()+'\t'
            text+='\n'
        return text

    def _parse_ascii_input(self, text):
        '''parse_ascii_input(self, text) --> list table

        Parses an ascii string to a parameter table. returns a list table if
        sucessful otherwise it returns None
        '''
        table=[]
        sucess=True
        lines=text.split('\n')
        for line in lines[:-1]:
            # If line not commented
            if line[0]!='#' and line[0]!='\n':
                line_strs=line.split('\t')
                # Check the format is it valid?
                if len(line_strs)>7 or len(line_strs)<6:
                    if len(line_strs)==1:
                        break
                    else:
                        sucess=False
                        break
                try:
                    par=line_strs[0]
                    val=float(line_strs[1])
                    fitted=line_strs[2].strip()=='True' \
                           or line_strs[2].strip()=='1'
                    min=float(line_strs[3])
                    max=float(line_strs[4])
                    error=line_strs[5]
                except Exception as e:
                    sucess=False
                    break
                else:
                    table.append([par, val, fitted, min, max, error])
        if sucess:
            return table
        else:
            return None

    def set_ascii_input(self, text):
        '''set_ascii_input(self, text) --> None

        If possible parse the text source and set the current parameters table
        to the one given in text.
        '''
        table=self._parse_ascii_input(text)
        if table:
            self.data=table
            return True
        else:
            return False

    def safe_copy(self, object):
        '''safe_copy(self, object) --> None

        Does a safe copy from object into this object.
        '''
        self.data=object.data[:]

    def copy(self):
        '''get_copy(self) --> copy of Parameters

        Does a copy of the current object.
        '''
        new_pars=Parameters()
        new_pars.data=self.data[:]

        return new_pars

    def __len__(self):
        return len([di for di in self.data if di[0].strip()!=''])

    def __getitem__(self, item):
        return ConnectedParameter(self, self.data[item])

    def __repr__(self):
        """
        Display information about the model.
        """
        output="Parameters:\n"
        output+="           "+" ".join(["%-16s"%label for label in self.data_labels])+"\n"
        for line in self.data:
            output+="           "+" ".join(["%-16s"%col for col in line])+"\n"
        return output

    def _repr_html_(self):
        output='<table><tr><th colspan="%i"><center>Parameters</center></th></tr>\n'%(len(self.data_labels)+1)
        output+="           <tr><th>No.</th><th>"+"</th><th>".join(
            ["%s"%label for label in self.data_labels])+"</th></tr>\n"
        for i, line in enumerate(self.data):
            output+="           <tr><td>%i</td><td>"%i
            output+="</td><td>".join(["%s"%col for col in line])+"\n"
        output+="</table>"
        return output

    @property
    def widget(self):
        return self._repr_ipyw_()

    def _repr_ipyw_(self):
        import ipywidgets as ipw
        vlist=[]
        header=ipw.HBox([ipw.HTML('<b>%s</b>'%txt[0], layout=ipw.Layout(width=txt[1])) for txt in
                         [('Parameter', '35%'), ('Value', '20%'), ('fit', '5%'),
                          ('min', '20%'), ('max', '20%')]])
        vlist.append(header)
        for par in self:
            vlist.append(par._repr_ipyw_())

        add_button=ipw.Button(description='Add Parameter')
        vlist.append(add_button)
        add_button.on_click(self._ipyw_add)
        vbox=ipw.VBox(vlist)
        add_button.vbox=vbox
        return vbox

    def _ipyw_add(self, button):
        self.append()

        prev_box=button.vbox.children
        button.vbox.children=prev_box[:-1]+(self[-1]._repr_ipyw_(), prev_box[-1])

class ConnectedParameter():
    """
    A representation of a single fittable parameter for use in api access to GenX.
    """

    def __init__(self, parent, data):
        self.data=data
        self._parent=parent

    @property
    def name(self):
        return self.data[0]

    @name.setter
    def name(self, value):
        model=self._parent.model
        if not model.is_compiled():
            model.compile_script()
        par_value=eval('self._parent.model.script_module.'+value.replace('.set', '.get')+'()',
                       globals(), locals())
        self.data[0]=value
        self.value=par_value
        self.min=0.25*par_value
        self.max=4.*par_value

    @property
    def value(self):
        return self.data[1]

    @value.setter
    def value(self, value):
        self.data[1]=float(value)

    @property
    def fit(self):
        return self.data[2]

    @fit.setter
    def fit(self, value):
        self.data[2]=bool(value)

    @property
    def min(self):
        return self.data[3]

    @min.setter
    def min(self, value):
        self.data[3]=float(value)

    @property
    def max(self):
        return self.data[4]

    @max.setter
    def max(self, value):
        self.data[4]=float(value)

    @property
    def error(self):
        return self.data[5]

    def __repr__(self):
        """
        Display information about the parameter.
        """
        output="Parameter:\n"
        output+="           "+" ".join(["%-16s"%label for label in self._parent.data_labels])+"\n"
        output+="           "+" ".join(["%-16s"%col for col in self.data])+"\n"
        return output

    def _repr_html_(self):
        output='<table><tr><th colspan="%i"><center>Parameter</center></th></tr>\n'%(len(self._parent.data_labels)+1)
        output+="           <tr><th>No.</th><th>"+"</th><th>".join(
            ["%s"%label for label in self._parent.data_labels])+"</th></tr>\n"
        output+="           <tr><td>%i</td><td>"%self._parent.data.index(self.data)
        output+="</td><td>".join(["%s"%col for col in self.data])+"\n"
        output+="</table>"
        return output

    def _repr_ipyw_(self):
        import ipywidgets as ipw
        wname=ipw.Text(value=self.name, layout=ipw.Layout(width='35%'))
        wname=ipw.Combobox(value=self.name,
                           options=[ni for ni, oi in self._parent.model.script_module.__dict__.items()
                                    if type(oi).__name__ in ['Layer', 'Stack', 'Instrument']])
        wname.change_item='name'
        wval=ipw.FloatText(value=self.value, layout=ipw.Layout(width='20%'))
        wval.change_item='value'
        wfit=ipw.Checkbox(value=self.fit, indent=False, layout=ipw.Layout(width='5%'))
        wfit.change_item='fit'
        wmin=ipw.FloatText(value=self.min, layout=ipw.Layout(width='20%'))
        wmin.change_item='min'
        wmax=ipw.FloatText(value=self.max, layout=ipw.Layout(width='20%'))
        wmax.change_item='max'
        entries=[wname, wval, wfit, wmin, wmax]
        wname.others=entries[1:]
        for entr in entries:
            entr.observe(self._ipyw_change, names='value')
        return ipw.HBox(entries)

    def _ipyw_change(self, change):
        if change.owner.change_item=='name':
            if '.' in change.new:
                try:
                    name=change.new.split('.')[0]
                    change.owner.options=tuple("%s.%s"%(name, si)
                                               for si in dir(self._parent.model.script_module.__dict__[name])
                                               if si.startswith('set'))
                except:
                    pass
            else:
                change.owner.options=tuple(ni for ni, oi in self._parent.model.script_module.__dict__.items()
                                           if type(oi).__name__ in ['Layer', 'Stack', 'Instrument'])

            if change.new=='':
                self.data[0]=''
                change.owner.description=''
                return
            prev=self.name
            try:
                self.name=change.new
            except Exception as err:
                self.data[0]=prev
                change.owner.description='ERR'
            else:
                change.owner.description=''
                change.owner.others[0].value=self.value
                change.owner.others[1].value=self.fit
                change.owner.others[2].value=self.min
                change.owner.others[3].value=self.max
        elif change.owner.change_item=='fit':
            self.fit=change.new
        elif change.owner.change_item=='value':
            self.value=change.new
        elif change.owner.change_item=='min':
            self.min=change.new
        elif change.owner.change_item=='min':
            self.min=change.new

if __name__=='__main__':
    p=Parameters()
    p.append()
    import h5py

    f=h5py.File('test.hdf', 'w')
    g=f.create_group('parameters')
    p.write_h5group(g)
    f.close()
    p2=Parameters()
    f=h5py.File('test.hdf', 'r')
    p2.read_h5group(f['parameters'])
    iprint(p2.data)



#!/usr/bin/env python
#
# -*- Mode: python -*-
#
# $Id: Simplex.py,v 1.2 2004/05/31 14:01:06 vivake Exp $
# 
# Copyright (c) 2002-2004 Vivake Gupta (vivakeATlab49.com).  All rights reserved.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# This software is maintained by Vivake (vivakeATlab49.com) and is available at:
#     http://shell.lab49.com/~vivake/python/Simplex.py
#
# 1.2  ( 5/2004) - Fixed a bug found by Noboru Yamamoto <noboru.yamamotoATkek.jp>
#                  which caused minimize() not to converge, and reach the maxiter
#                  limit under some conditions.


""" Simplex - a regression method for arbitrary nonlinear function minimization

Simplex minimizes an arbitrary nonlinear function of N variables by the
Nedler-Mead Simplex method as described in:

Nedler, J.A. and Mead, R. "A Simplex Method for Function Minimization." 
    Computer Journal 7 (1965): 308-313.

It makes no assumptions about the smoothness of the function being minimized.
It converges to a local minimum which may or may not be the global minimum
depending on the initial guess used as a starting point.
"""

import math
import copy
from genx.gui_logging import iprint

class Simplex:
    def __init__(self, testfunc, guess, increments, kR=-1, kE=2, kC=0.5):
        """Initializes the simplex.
        INPUTS
        ------
        testfunc      the function to minimize
        guess[]       an list containing initial guesses
        increments[]  an list containing increments, perturbation size
        kR            reflection constant
        kE            expansion constant
        kC            contraction constant
        """
        self.testfunc=testfunc
        self.guess=guess
        self.increments=increments
        self.kR=kR
        self.kE=kE
        self.kC=kC
        self.numvars=len(self.guess)
        self.simplex=[]

        self.lowest=-1
        self.highest=-1
        self.secondhighest=-1

        self.errors=[]
        self.currenterror=0

        # Initialize vertices
        for vertex in range(0, self.numvars+3):  # Two extras to store centroid and reflected point
            self.simplex.append(copy.copy(self.guess))
        # Use initial increments
        for vertex in range(0, self.numvars+1):
            for x in range(0, self.numvars):
                if x==(vertex-1):
                    self.simplex[vertex][x]=self.guess[x]+self.increments[x]
            self.errors.append(0)
        self.calculate_errors_at_vertices()

    def minimize(self, epsilon=0.0001, maxiters=250, monitor=0):
        """Walks to the simplex down to a local minima.
        INPUTS
        ------
        epsilon       convergence requirement
        maxiters      maximum number of iterations
        monitor       if non-zero, progress info is output to stdout  

        OUTPUTS
        -------
        an array containing the final values
        lowest value of the error function
        number of iterations taken to get here
        """
        iter=0
        for iter in range(0, maxiters):
            # Identify highest, second highest, and lowest vertices
            self.highest=0
            self.lowest=0
            for vertex in range(0, self.numvars+1):
                if self.errors[vertex]>self.errors[self.highest]:
                    self.highest=vertex
                if self.errors[vertex]<self.errors[self.lowest]:
                    self.lowest=vertex
            self.secondhighest=0
            for vertex in range(0, self.numvars+1):
                if vertex==self.highest:
                    continue
                if self.errors[vertex]>self.errors[self.secondhighest]:
                    self.secondhighest=vertex
            # Test for convergence
            S=0.0
            S1=0.0
            for vertex in range(0, self.numvars+1):
                S=S+self.errors[vertex]
            F2=S/(self.numvars+1)
            for vertex in range(0, self.numvars+1):
                S1=S1+(self.errors[vertex]-F2)**2
            T=math.sqrt(S1/self.numvars)

            # Optionally, print progress information
            if monitor:
                iprint('Iteration = %d   Best = %f   Worst = %f'%(
                iter, self.errors[self.lowest], self.errors[self.highest]))

            if T<=epsilon:  # We converged!  Break out of loop!
                break
            else:  # Didn't converge.  Keep crunching.
                # Calculate centroid of simplex, excluding highest vertex
                for x in range(0, self.numvars):
                    S=0.0
                    for vertex in range(0, self.numvars+1):
                        if vertex==self.highest:
                            continue
                        S=S+self.simplex[vertex][x]
                    self.simplex[self.numvars+1][x]=S/self.numvars

                self.reflect_simplex()

                self.currenterror=self.testfunc(self.guess)

                if self.currenterror<self.errors[self.lowest]:
                    tmp=self.currenterror
                    self.expand_simplex()
                    self.currenterror=self.testfunc(self.guess)
                    if self.currenterror<tmp:
                        self.accept_expanded_point()
                    else:
                        self.currenterror=tmp
                        self.accept_reflected_point()

                elif self.currenterror<=self.errors[self.secondhighest]:
                    self.accept_reflected_point()

                elif self.currenterror<=self.errors[self.highest]:
                    self.accept_reflected_point()

                    self.contract_simplex()
                    self.currenterror=self.testfunc(self.guess)
                    if self.currenterror<self.errors[self.highest]:
                        self.accept_contracted_point()
                    else:
                        self.multiple_contract_simplex()

                elif self.currenterror>=self.errors[self.highest]:
                    self.contract_simplex()
                    self.currenterror=self.testfunc(self.guess)
                    if self.currenterror<self.errors[self.highest]:
                        self.accept_contracted_point()
                    else:
                        self.multiple_contract_simplex()

        # Either converged or reached the maximum number of iterations.
        # Return the lowest vertex and the currenterror.
        for x in range(0, self.numvars):
            self.guess[x]=self.simplex[self.lowest][x]
        self.currenterror=self.errors[self.lowest]
        return self.guess, self.currenterror, iter

    def contract_simplex(self):
        for x in range(0, self.numvars):
            self.guess[x]=self.kC*self.simplex[self.highest][x]+(1-self.kC)*self.simplex[self.numvars+1][x]
        return

    def expand_simplex(self):
        for x in range(0, self.numvars):
            self.guess[x]=self.kE*self.guess[x]+(1-self.kE)*self.simplex[self.numvars+1][x]
        return

    def reflect_simplex(self):
        for x in range(0, self.numvars):
            self.guess[x]=self.kR*self.simplex[self.highest][x]+(1-self.kR)*self.simplex[self.numvars+1][x]
            self.simplex[self.numvars+2][x]=self.guess[x]  # REMEMBER THE REFLECTED POINT
        return

    def multiple_contract_simplex(self):
        for vertex in range(0, self.numvars+1):
            if vertex==self.lowest:
                continue
            for x in range(0, self.numvars):
                self.simplex[vertex][x]=0.5*(self.simplex[vertex][x]+self.simplex[self.lowest][x])
        self.calculate_errors_at_vertices()
        return

    def accept_contracted_point(self):
        self.errors[self.highest]=self.currenterror
        for x in range(0, self.numvars):
            self.simplex[self.highest][x]=self.guess[x]
        return

    def accept_expanded_point(self):
        self.errors[self.highest]=self.currenterror
        for x in range(0, self.numvars):
            self.simplex[self.highest][x]=self.guess[x]
        return

    def accept_reflected_point(self):
        self.errors[self.highest]=self.currenterror
        for x in range(0, self.numvars):
            self.simplex[self.highest][x]=self.simplex[self.numvars+2][x]
        return

    def calculate_errors_at_vertices(self):
        for vertex in range(0, self.numvars+1):
            if vertex==self.lowest:
                continue
            for x in range(0, self.numvars):
                self.guess[x]=self.simplex[vertex][x]
            self.currenterror=self.testfunc(self.guess)
            self.errors[vertex]=self.currenterror
        return

def objective_function(args):
    return abs(args[0]*args[0]*args[0]*5-args[1]*args[1]*7+math.sqrt(abs(args[0]))-118)

def main():
    s=Simplex(objective_function, [1, 1, 1], [2, 4, 6])
    values, err, iter=s.minimize(maxiters=60, monitor=0)
    iprint('args = ', values)
    iprint('error = ', err)
    iprint('iterations = ', iter)

if __name__=='__main__':
    main()



'''Library that implements an effective density model for composition 
profiles (as described by Tolan) and also a home constructed model for
magnetic moment profiles (element specific). 
'''
# from pylab import *
from numpy import *

import numpy as np

from scipy.special import erf

def erf_profile(z, z0, d, sigma0, sigma1):
    eta=(sigma1*z0+sigma0*(z0+d))/(sigma0+sigma1)
    p=0.5*(1+where(z<=eta, erf((z-z0)/sqrt(2.)/sigma0),
                   -erf((z-z0-d)/sqrt(2.)/sigma1)))
    return p

def erf_interf(z, sigma):
    return 0.5+0.5*erf(z/sqrt(2.)/sigma)

def _lin(z, sigma):
    p=zeros(z.shape)
    p=((z>sqrt(3)*sigma)*1.0+
       (abs(z)<=sqrt(3)*sigma)*(0.5+z/2/sqrt(3)/sigma))
    return p

def lin_profile(z, z0, d, sigma0, sigma1):
    eta=(sigma1*z0+sigma0*(z0+d))/(sigma0+sigma1)
    p=where(z<eta, _lin(z-z0, sigma0),
            _lin(-(z-z0-d), sigma1))
    return p

def _exp(z, sigma):
    p=((z<=0)*(0.5*exp(sqrt(2)*z/sigma))+
       (z>0)*(1-0.5*exp(-sqrt(2)*z/sigma)))
    return p

def exp_profile(z, z0, d, sigma0, sigma1):
    eta=(sigma1*z0+sigma0*(z0+d))/(sigma0+sigma1)
    p=where(z<eta, _exp(z-z0, sigma0),
            _exp(-(z-z0-d), sigma1))
    return p

def _sin(z, sigma):
    a=pi/sqrt(pi**2-8)*sigma
    p=(z>a)*1.0+(abs(z)<=a)*(0.5+0.5*sin(pi*z/2/a))
    return p

def sin_profile(z, z0, d, sigma0, sigma1):
    eta=(sigma1*z0+sigma0*(z0+d))/(sigma0+sigma1)
    p=where(z<eta, _sin(z-z0, sigma0),
            _sin(-(z-z0-d), sigma1))
    return p

def compress_profile_old(z, p, delta_max):
    pnew=p.copy()
    znew=z.copy()
    i=0
    index=array([False])
    while any(bitwise_not(index)):
        # print i
        i+=1
        index=abs(pnew[:-1]-pnew[1:])>delta_max
        index[::2]=True
        index=r_[index, True]
        # print pnew.shape, index.shape
        pnew=pnew[index]
        znew=znew[index]

    return znew, pnew

def compress_profile(z, p, delta_max):
    ''' Compresses a profile by merging the neighbouring layers that
    has an density difference of maximum delta_max'''
    pnew=p.copy()
    znew=z.copy()
    inew=arange(len(p))
    i=0
    index=array([False])
    while any(bitwise_not(index)):
        # print i
        i+=1
        index=array([True]*len(pnew))
        index[1:-1:2]=abs(pnew[:-2:2]-pnew[2::2])>delta_max
        # print pnew.shape, index.shape
        pnew=pnew[index]
        znew=znew[index]
        inew=inew[index]
    # print inew
    inew=append(inew, inew[-1])
    pret=array([p[inew[i]:inew[i+1]+1].mean() for i in range(len(inew)-1)])
    return znew, pret

def compress_profile2(z, p1, p2, delta_max):
    ''' Compresses a profile by merging the neighbouring layers that
    has an density difference of maximum delta_max'''
    p1new=p1.copy()
    p2new=p2.copy()
    znew=z.copy()
    inew=arange(len(p1))
    i=0
    index=array([False])
    while any(bitwise_not(index)):
        # print i
        i+=1
        index=array([True]*len(p1new))
        index[1:-1:2]=bitwise_or(abs(p1new[:-2:2]-p1new[2::2])>delta_max
                                 , abs(p2new[:-2:2]-p2new[2::2])>delta_max
                                 )
        # print pnew.shape, index.shape
        p1new=p1new[index]
        p2new=p2new[index]
        znew=znew[index]
        inew=inew[index]
    # print inew
    inew=append(inew, inew[-1])
    p1ret=array([p1[inew[i]:inew[i+1]+1].mean() for i in range(len(inew)-1)])
    p2ret=array([p2[inew[i]:inew[i+1]+1].mean() for i in range(len(inew)-1)])
    return znew, p1ret, p2ret

def compress_profile_n(z, ps, delta_maxs):
    ''' Compresses multiple profiles, each profile: ps[i], 
    by merging the neighbouring layers that
    has an density difference of maximum delta_maxs[i]'''
    inew, znew=compress_profile_index_n(z, ps, delta_maxs)
    psnew=create_compressed_profile(ps, inew)
    return znew, psnew

def compare_layers(ps, delta_max):
    """Compare consecutive layers and see if ps is smaller than delta_max"""
    if len(ps.shape)>1:
        return (abs(ps[:-2:2]-ps[2::2])>delta_max).any(axis=1)
    else:
        return abs(ps[:-2:2]-ps[2::2])>delta_max

def compress_profile_index_n(z, ps, delta_maxs):
    znew=z.copy()
    inew=arange(len(ps[0]))
    psnew=[ps_i.copy() for ps_i in ps]
    i=0
    index=array([False])
    while any(bitwise_not(index)):
        i+=1
        index=array([True]*len(psnew[0]))
        test=array([compare_layers(ps_i, delta_maxs_i) for ps_i, delta_maxs_i in zip(psnew, delta_maxs)])
        index[1:-1:2]=test.any(axis=0)

        psnew=[ps_i[index] for ps_i in psnew]
        znew=znew[index]
        inew=inew[index]
    inew=append(inew, inew[-1])
    return inew, znew

def create_compressed_profile(ps, inew):
    psret=[array([ps_i[inew[i]:inew[i+1]+1].mean(0) for i in range(len(inew)-1)]) for ps_i in ps]
    return psret

def create_profile(d, sigma, dens, prof_funcs, dz=0.01,
                   mult=3, buffer=0, delta=20):
    zlay=r_[0, cumsum(d)]
    z=arange(-sigma[0]*mult-buffer, zlay[-1]+sigma[-1]*mult+buffer, dz)
    ptot=z*0
    pdens=z*0
    ps=[]
    d=r_[delta+sigma[0]*mult+buffer, d, sigma[-1]*mult+delta+buffer]
    sigma=r_[0, sigma, 0]
    zlay=cumsum(d)
    # print zlay
    z0=buffer+sigma[1]*mult
    zlay=r_[0, zlay]-z0
    # print zlay
    # for i in range(len(d)):
    #    print z0
    #    p = make_profile(z, z0, d[i], sigma[i], sigma[i+1])
    #    z0 += d[i]
    #    ptot += p
    #    pdens += p*dens[i]
    #    ps.append(p)
    ps=array([prof(z, zp, di, s_low, s_up)
              for prof, zp, di, s_low, s_up in zip(prof_funcs, zlay,
                                                   d, sigma[:-1], sigma[1:])])
    ptot=ps.sum(0)
    pdens_indiv=(ps*dens[:, newaxis])/ptot
    pdens=pdens_indiv.sum(0)

    return z, pdens, pdens_indiv

def create_profile_cm(d, sigma_c, sigma_m, prof_funcs,
                      prof_funcs_mag, dmag_dens_l, dmag_dens_u, mag_dens, dd_m,
                      dz=0.01, mult=3, buffer=0, delta=20):
    '''Create a scattering length profile for magnetic x-ray scattering for the charge and
    magnetic part of the scattering lengths. sigma is roughnesses.
    Note that prof_funcs relate to layer profiles and prof_funcs_mag relates to 
    magnetic interface profiles.
    '''
    zlay=r_[0, cumsum(d)]
    s_max_bot=max(sigma_c[0], sigma_m[0])
    s_max_up=max(sigma_c[-1], sigma_m[-1])
    z=arange(-s_max_bot*mult-buffer, zlay[-1]+s_max_up*mult+buffer+dz, dz)
    ptot=z*0
    pdens=z*0
    ps=[]
    d=r_[delta+s_max_bot*mult+buffer, d, s_max_up*mult+delta+buffer]
    sigma_c=r_[0, sigma_c, 0]+1e-20
    sigma_m=r_[0, sigma_m, 0]+1e-20
    dd_m=r_[0, dd_m, 0]+1e-20
    prof_funcs_mag=[prof_funcs_mag[0]]+prof_funcs_mag+[prof_funcs_mag[-1]]
    zlay=cumsum(d)
    z0=delta+buffer+s_max_bot*mult
    zlay=r_[0, zlay]-z0
    ps=array([r_[prof(z, zp, di, s_c_low, s_c_up),
                 # prof(z, zp, di, s_m_low, s_m_up)]
                 prof_m_l(-(z-zp-dd_low), s_m_low)*dm_l+prof_m_u(z-zp-di-dd_up, s_m_up)*dm_u+m]
              for prof, zp, di, s_c_low, s_c_up, s_m_low, s_m_up,
                  prof_m_l, prof_m_u, dm_l, dm_u, m, dd_low, dd_up
              in zip(prof_funcs, zlay, d, sigma_c[:-1], sigma_c[1:],
                     sigma_m[:-1], sigma_m[1:],
                     prof_funcs_mag[:-1], prof_funcs_mag[1:],
                     dmag_dens_l, dmag_dens_u, mag_dens, dd_m[:-1], dd_m[1:]
                     )])
    ptot=ps.sum(0)
    # print ps.shape, ptot.shape
    pdens_indiv=(ps[:, :len(z)])/ptot[:len(z)]
    pdens_indiv_c=pdens_indiv  # *dens_c[:,newaxis]
    pdens_indiv_m=ps[:, len(z):]  # *dens_m[:,newaxis]
    # pdens_c = pdens_indiv_c.sum(0)
    # pdens_m = (pdens_indiv_m*pdens_indiv).sum(0)

    return z, pdens_indiv_c, pdens_indiv_m

def create_profile_cm2(d, sigma_c, sigma_ml, sigma_mu, prof_funcs,
                       prof_funcs_mag, dmag_dens_l, dmag_dens_u, mag_dens, dd_l, dd_u,
                       dz=0.01, mult=3, buffer=0, delta=20):
    '''Create a scattering length profile for magnetic x-ray scattering for the charge and
    magnetic part of the scattering lengths. sigma is roughnesses.
    Note that prof_funcs relate to layer profiles and prof_funcs_mag relates to 
    magnetic interface profiles.
    '''
    zlay=r_[0, cumsum(d)]
    s_max_bot=max(sigma_c[0], sigma_ml[0])
    s_max_up=max(sigma_c[-1], sigma_mu[-1])
    z=arange(-s_max_bot*mult-buffer, zlay[-1]+s_max_up*mult+buffer+dz, dz)
    ptot=z*0
    pdens=z*0
    ps=[]
    d=r_[delta+s_max_bot*mult+buffer, d, s_max_up*mult+delta+buffer]
    sigma_c=r_[0, sigma_c, 0]+1e-20
    sigma_mu=r_[sigma_mu, 0]+1e-20
    sigma_ml=r_[0, sigma_ml]+1e-20

    dd_l=r_[dd_l, 0]+1e-20
    dd_u=r_[dd_u, 0]+1e-20

    prof_funcs_mag=[prof_funcs_mag[0]]+prof_funcs_mag+[prof_funcs_mag[-1]]
    zlay=cumsum(d)
    z0=delta+buffer+s_max_bot*mult
    zlay=r_[0, zlay]-z0
    ps=array([r_[prof(z, zp, di, s_c_low, s_c_up),
                 # prof(z, zp, di, s_m_low, s_m_up)]
                 prof_m_l(-(z-zp-dd_low), s_m_low)*dm_l*m+prof_m_u((z-zp-di+dd_up), s_m_up)*dm_u*m+m]
              for prof, zp, di, s_c_low, s_c_up, s_m_low, s_m_up,
                  prof_m_l, prof_m_u, dm_l, dm_u, m, dd_low, dd_up
              in zip(prof_funcs, zlay, d, sigma_c[:-1], sigma_c[1:],
                     sigma_ml, sigma_mu,
                     prof_funcs_mag[:-1], prof_funcs_mag[1:],
                     dmag_dens_l, dmag_dens_u, mag_dens, dd_l, dd_u
                     )])
    ptot=ps.sum(0)
    # print ps.shape, ptot.shape
    pdens_indiv=(ps[:, :len(z)])/ptot[:len(z)]
    pdens_indiv_c=pdens_indiv  # *dens_c[:,newaxis]
    pdens_indiv_m=ps[:, len(z):]  # *dens_m[:,newaxis]
    # pdens_c = pdens_indiv_c.sum(0)
    # pdens_m = (pdens_indiv_m*pdens_indiv).sum(0)

    return z, pdens_indiv_c, pdens_indiv_m

if __name__=='__main__':
    from pylab import *
    from mpl_toolkits.axes_grid import Grid
    from mpl_toolkits.axes_grid.anchored_artists import AnchoredText

    # sigma0 = 5.
    # sigma1 = 1.
    # mult = 3
    # buffer = 0
    # delta = 20
    # z0 = sigma0*mult
    # d = 9.#sigma0*mult + sigma1*mult
    # z = arange(0, z0 + d + sigma1*mult, 0.1)
    # p = make_profile(z, z0, d, sigma0, sigma1)
    # plot(z,p)
    sigma_c=array([4, 4, 2])*1.0  # sigma_c = array([2,8,2])*1.0
    sigma_m=array([1, 1, 1])*1.0  # sigma_m = array([2,5,4])*1.0
    dmag_l=array([0.0, 0.0, 0.0])
    dmag_u=array([0.0, 0.0, 0.0])
    mag_dens=array([0.0, 2.25, 0.0])
    # ddm = array([10.0, -10.0, 0.0])
    ddm=array([10.0, -10.0, 10.0])
    d=array([0.0, 50, 0.0])*1.0  # d = array([5, 60])*1.0
    dens_c=array([-1.79e-3+0.0029J, -4e-23])  # dens_c = array([5,50,25,0])
    dens_m=array([1.4458e-3-0.00144J, 2.12e-23])  # dens_m = array([0,6,10,0])
    prof_funcs=[erf_profile]*3  # prof_funcs = [erf_profile]*4
    mag_prof_funcs=[erf_interf]*3
    # z, pdens, pdens_indiv = create_profile(d, sigma_c, dens_cprof_funcs, dz = 0.1,
    #                                       mult = 5.0, 
    #                                       buffer = 0, delta = 5.)
    z, c_den, m_den=create_profile_cm(d[1:-1], sigma_c[:-1], sigma_m[:-1],
                                      prof_funcs, mag_prof_funcs,
                                      dmag_l, dmag_u, mag_dens,
                                      ddm, dz=1.0, mult=4.0,
                                      buffer=20, delta=5.)

    def plot_fig(filename, fig_title=''):
        fig=figure(1)
        grid=Grid(fig, 111, nrows_ncols=(3, 1), axes_pad=0.15)
        # subplot(311)
        grid[0].set_title(fig_title)
        grid[0].plot(z, c_den[0], lw=2.0, c='b')
        grid[0].plot(z, c_den[1], lw=2.0, c='r')
        grid[0].plot(z, c_den[2], lw=2.0, c='g')
        grid[0].axvline(0, linestyle='--', c='k')
        grid[0].axvline(d[1], linestyle='--', c='k')
        grid[0].set_ylabel('Comp.')
        grid[0].legend(('Substrate', 'Fe layer', 'Ambient'))
        # subplot(312)
        grid[1].plot(z, m_den[1], lw=2.0, c='r')
        grid[1].axvline(0, linestyle='--', c='k')
        grid[1].axvline(d[1], linestyle='--', c='k')
        grid[1].axvline(0+ddm[0], linestyle=':', c='k')
        grid[1].axvline(d[1]+ddm[1], linestyle=':', c='k')
        # grid[1].annotate('Sub.dd_m = %.1f'%ddm[0], xy=(ddm[0], mag_dens[1]), xycoords = 'data',
        #                 xytext = (-50, 0), textcoords = 'offset points', 
        #                 arrowprops=dict(arrowstyle="->", connectionstyle = 'arc3'))
        # grid[1].annotate('Fe.dd_m = %.1f'%ddm[1], xy=(d[1]+ddm[1], mag_dens[1]), xycoords = 'data',
        #                 xytext = (50, 0), textcoords = 'offset points', 
        #                 arrowprops=dict(arrowstyle="<-", connectionstyle = 'arc3'))
        at=AnchoredText("dd_ml = %.1f"%(ddm[0]),
                        prop=dict(size=10), frameon=True,
                        loc=6,
                        )
        at.patch.set_boxstyle("round,pad=0.,rounding_size=0.2")
        grid[1].add_artist(at)
        at=AnchoredText("dd_mu = %.1f"%(ddm[1]),
                        prop=dict(size=10), frameon=True,
                        loc=7,
                        )
        at.patch.set_boxstyle("round,pad=0.,rounding_size=0.2")
        grid[1].add_artist(at)
        grid[1].set_ylabel('Mag. mom.\n per res. atom.')
        grid[1].set_ylim(0, 3.1)
        # subplot(313)
        grid[2].plot(z, m_den[1]*c_den[1], 'r', z, 2.25*c_den[1], 'b', lw=2.0)
        grid[2].axvline(0, linestyle='--', c='k')
        grid[2].axvline(d[1], linestyle='--', c='k')
        grid[2].axvline(0+ddm[0], linestyle=':', c='k')
        grid[2].axvline(d[1]+ddm[1], linestyle=':', c='k')
        grid[2].set_ylabel('Tot. Mag mom.')
        grid[2].legend(('Profile', 'Bulk Fe'))
        grid[2].set_xlabel('Depth [AA]')
        savefig(filename)
        clf()

    plot_fig('Bulk_Fe_sigma', 'Bulk Fe with no change at the interfaces')
    dmag_l=array([0.0, 0.75, 0.0])
    dmag_u=array([0.0, 0.75, 0.0])
    z, c_den, m_den=create_profile_cm(d[1:-1], sigma_c[:-1], sigma_m[:-1],
                                      prof_funcs, mag_prof_funcs,
                                      dmag_l, dmag_u, mag_dens,
                                      ddm, dz=1.0, mult=4.0,
                                      buffer=20, delta=5.)
    plot_fig('Enhanced_Fe_sigma', 'Bulk Fe with enhanced interfaces')
    dmag_l=array([0.0, -1., 0.0])
    dmag_u=array([0.0, -1., 0.0])
    z, c_den, m_den=create_profile_cm(d[1:-1], sigma_c[:-1], sigma_m[:-1],
                                      prof_funcs, mag_prof_funcs,
                                      dmag_l, dmag_u, mag_dens,
                                      ddm, dz=1.0, mult=4.0,
                                      buffer=20, delta=5.)
    plot_fig('Depleted_Fe_sigma', 'Bulk Fe with depleted interfaces')



"""
Help class for the Reflectivity plugin to analyze and change the script.
"""
import re

avail_models=['spec_nx', 'spec_inhom', 'spec_adaptive',
              'interdiff', 'mag_refl', 'soft_nx']

def default_html_decorator(name, str):
    return str

class SampleHandler:
    def __init__(self, sample, names):
        self.sample=sample
        self.names=names
        self.getStringList()

    def getStringList(self, html_encoding=False, html_decorator=default_html_decorator):
        '''
        Function to generate a list of strings that gives
        a visual representation of the sample.
        '''
        slist=[self.sample.Substrate.__repr__()]
        poslist=[(None, None)]
        i=0;
        j=0
        for stack in self.sample.Stacks:
            j=0
            for layer in stack.Layers:
                slist.append(layer.__repr__())
                poslist.append((i, j))
                j+=1
            slist.append('Stack: Repetitions = %s'%str(stack.Repetitions))
            for key in list(stack._parameters.keys()):
                if not key in ['Repetitions', 'Layers']:
                    slist[-1]+=', %s = %s'%(key, str(getattr(stack, key)))
            poslist.append((i, None))
            i+=1
        slist.append(self.sample.Ambient.__repr__())
        for item in range(len(slist)):
            name=self.names[-item-1]
            par_str=slist[item]
            if slist[item][0]=='L' and item!=0 and item!=len(slist)-1:
                if html_encoding:
                    slist[item]=('<code>&nbsp;&nbsp;&nbsp;<b>'+name+'</b> = '
                                 +html_decorator(name, par_str)+'</code>')
                else:
                    slist[item]=self.names[-item-1]+' = model.'+slist[item]
            else:
                if item==0 or item==len(slist)-1:
                    # This is then the ambient or substrates
                    if html_encoding:
                        slist[item]=('<code><b>'+name+'</b> = '
                                     +html_decorator(name, par_str)+'</code>')
                    else:
                        slist[item]=self.names[-item-1]+' = model.'+slist[item]
                else:
                    # This is a stack!
                    if html_encoding:
                        slist[item]=('<font color = "BLUE"><code><b>'+name+'</b> = '
                                     +html_decorator(name, par_str)+'</code></font>')
                    else:
                        slist[item]=self.names[-item-1]+' = model.'+slist[item]
        poslist.append((None, None))
        slist.reverse()
        poslist.reverse()
        self.poslist=poslist
        return slist

    def htmlize(self, code):
        '''htmlize(self, code) --> code

        htmlize the code for display
        '''
        p=code.index('=')
        name='<code><b>%s</b></code>'%code[:p]
        items=code[p:].split(',')
        return name+''.join(['<code>%s,</code>'%item for item in items])

    def getCode(self):
        '''
        Generate the python code for the current sample structure.
        '''
        slist=self.getStringList()
        layer_code=''

        # Create code for the layers:
        for item in slist:
            if item.find('Layer')>-1:
                itemp=item.lstrip()
                layer_code=layer_code+itemp+'\n'
        # Create code for the Stacks:
        i=0
        stack_code=''
        item=slist[i]
        maxi=len(slist)-1
        while (i<maxi):
            if item.find('Stack')>-1:
                stack_strings=item.split(':')
                stack_code=stack_code+stack_strings[0]+'(Layers=['
                i+=1
                item=slist[i]
                stack_layers=[]
                while (item.find('Stack')<0 and i<maxi):
                    itemp=item.split('=')[0]
                    itemp=itemp.lstrip()
                    stack_layers.append(itemp)
                    # stack_code = stack_code + itemp+','
                    i+=1
                    item=slist[i]
                stack_layers.reverse()
                stack_code+=', '.join(stack_layers)
                i-=1
                if stack_code[-1]!='[':
                    stack_code=stack_code[:-1]+'],'+stack_strings[1]+')\n'
                else:
                    stack_code=stack_code[:]+'],'+stack_strings[1]+')\n'
            i+=1
            item=slist[i]
        # Create the code for the sample
        sample_code='sample = model.Sample(Stacks = ['
        stack_strings=stack_code.split('\n')
        rest_sample_rep='], '
        sample_string_pars=self.sample.__repr__().split(':')[1].split('\n')[0].lstrip()
        if len(sample_string_pars)!=0:
            sample_string_pars+=', '
        rest_sample_rep+=sample_string_pars+'Ambient = Amb, Substrate = Sub)\n'
        if stack_strings!=['']:
            # Added 20080831 MB bugfix
            stack_strings.reverse()
            for item in stack_strings[1:]:
                itemp=item.split('=')[0]
                sample_code=sample_code+itemp+','
            sample_code=sample_code[:-2]+rest_sample_rep
        else:
            sample_code+=rest_sample_rep

        return layer_code, stack_code, sample_code

    def getItem(self, pos):
        '''
        Returns the item (Stack or Layer) at position pos
        '''
        if pos==0:
            return self.sample.Ambient
        if pos==len(self.poslist)-1:
            return self.sample.Substrate
        stack=self.sample.Stacks[self.poslist[pos][0]]
        if self.poslist[pos][1] is None:
            return stack
        return stack.Layers[self.poslist[pos][1]]

    def __getitem__(self, item):
        if item in self.names:
            return self.getItem(self.names.index(item))
        elif type(item) is int:
            return self.getItem(item)
        else:
            raise IndexError('Item has to be in .names list or an integer')

    def __delitem__(self, key):
        return self.deleteItem(key)

    def __len__(self):
        return len(self.names)

    def deleteItem(self, pos):
        '''
        Delete item pos in the lsit if the item is a stack all the Layers
        are deleted as well.
        '''
        if pos in self.names:
            pos=self.names.index(pos)

        if pos==0:
            return None
        if pos==len(self.poslist)-1:
            return None
        stack=self.sample.Stacks[self.poslist[pos][0]]
        if self.poslist[pos][1] is None:
            self.sample.Stacks.pop(self.poslist[pos][0])
            p=self.poslist[pos][0]
            pt=pos
            while self.poslist[pt][0]==p:
                pt+=1
            pt-=1
            while self.poslist[pt][0]==p:
                self.names.pop(pt)
                pt-=1

        else:
            stack.Layers.pop(self.poslist[pos][1])
            self.names.pop(pos)
        return self.getStringList()

    def insertItem(self, pos, type, name='test'):
        '''
        Insert an item into the sample at position pos in the list
        and of type. type is a string of either Stack or Layer
        '''
        if pos in self.names:
            pos=self.names.index(pos)

        spos=self.poslist[pos]
        added=False
        last=False
        if pos==0:
            spos=(self.poslist[1][0], self.poslist[1][1])  # +1
            # spos=(None,None)
        if pos==len(self.poslist)-1:
            spos=self.poslist[-2]
            last=True
        stackp=False
        if spos[1] is None:
            spos=(spos[0], 0)
            stackp=True
        if spos[0] is None:
            spos=(0, spos[1])

        # If it not the first item i.e. can't insert anything before the
        # ambient layer
        if pos!=0:
            if type=='Stack':
                stack=self.model.Stack(Layers=[])
                if last:
                    self.names.insert(pos, name)
                else:
                    if stackp:
                        self.names.insert(pos+len(self.sample.Stacks[spos[0]].Layers)+1, name)
                    else:
                        self.names.insert(pos+spos[1]+1, name)
                self.sample.Stacks.insert(spos[0], stack)
                added=True

            if type=='Layer' and len(self.poslist)>2:
                layer=self.model.Layer()
                if last:
                    self.names.insert(pos, name)
                else:
                    if spos[1]>=0:
                        self.names.insert(pos+1, name)
                    else:
                        self.names.insert(pos+len(self.sample.Stacks[spos[0]].Layers)+1, name)
                if last:
                    self.sample.Stacks[spos[0]].Layers.insert(0, layer)
                else:
                    if self.poslist[pos][1] is None:
                        self.sample.Stacks[spos[0]].Layers.append(layer)
                    else:
                        self.sample.Stacks[spos[0]].Layers.insert(spos[1], layer)
                added=True

        else:
            if type=='Stack':
                stack=self.model.Stack(Layers=[])
                self.sample.Stacks.append(stack)
                added=True
                self.names.insert(pos+1, name)
            if type=='Layer' and len(self.poslist)>2:
                layer=self.model.Layer()
                self.sample.Stacks[spos[0]].Layers.append(layer)
                added=True
                self.names.insert(pos+2, name)
        if added:
            return self.getStringList()
        else:
            return None

    def canInsertLayer(self):
        return self.poslist>2

    def checkName(self, name):
        return self.names.__contains__(name)

    def getName(self, pos):
        """ Returns the name for the object at pos
        :param pos: list position for the name
        :return: the name (string)
        """
        return self.names[pos]

    def changeName(self, pos, name):
        if pos in self.names:
            pos=self.names.index(pos)
        if name in self.names and name!=self.names[pos]:
            return False
        elif pos==len(self.names)-1 or pos==0:
            return False
        else:
            self.names[pos]=name
            return True

    def moveUp(self, pos):
        '''
        Move the item up - with stacks move the entire stack up one step.
        Moves layer only if it is possible.
        '''
        if pos in self.names:
            pos=self.names.index(pos)
        if pos>1 and pos!=len(self.poslist)-1:
            if self.poslist[pos][1] is None:
                temp=self.sample.Stacks.pop(self.poslist[pos][0])
                temps=[]
                for index in range(len(temp.Layers)+1):
                    temps.append(self.names.pop(pos))
                for index in range(len(temp.Layers)+1):
                    self.names.insert(pos-len(self.sample.Stacks[self.poslist[pos][0]].Layers)-1, temps[-index-1])
                self.sample.Stacks.insert(self.poslist[pos][0]+1, temp)
                return self.getStringList()
            else:  # i.e. it is a layer we move
                if pos>2:
                    temp=self.sample.Stacks[self.poslist[pos][0]].Layers.pop(self.poslist[pos][1])
                    temps=self.names.pop(pos)
                    if self.poslist[pos-1][1] is None:  # Next item a Stack i.e. jump up
                        self.sample.Stacks[self.poslist[pos-2][0]].Layers.insert(0, temp)
                        self.names.insert(pos-1, temps)
                    else:  # Moving inside a stack
                        self.sample.Stacks[self.poslist[pos][0]].Layers.insert(self.poslist[pos][1]+1, temp)
                        self.names.insert(pos-1, temps)
                    return self.getStringList()
                else:
                    return None
        else:
            return None

    def moveDown(self, pos):
        '''
        Move the item down - with stacks move the entire stack up one step.
        Moves layer only if it is possible.
        '''
        if pos in self.names:
            pos=self.names.index(pos)

        if pos!=0 and pos<len(self.poslist)-2:
            if self.poslist[pos][1] is None:  # Moving a stack
                if self.poslist[pos][0]!=0:
                    temp=self.sample.Stacks.pop(self.poslist[pos][0])
                    temps=[]
                    for index in range(len(temp.Layers)+1):
                        temps.append(self.names.pop(pos))
                    for index in range(len(temp.Layers)+1):
                        self.names.insert(pos+len(self.sample.Stacks[self.poslist[pos][0]-1].Layers)+1, temps[-index-1])
                    self.sample.Stacks.insert(self.poslist[pos][0]-1, temp)
                    return self.getStringList()
                else:
                    return None

            else:  # i.e. it is a layer we move
                if pos<len(self.poslist)-2:
                    temp=self.sample.Stacks[self.poslist[pos][0]].Layers.pop(self.poslist[pos][1])
                    temps=self.names.pop(pos)
                    if self.poslist[pos+1][1] is None:  # Next item a Stack i.e. jump down
                        self.sample.Stacks[self.poslist[pos+1][0]].Layers.insert(
                            len(self.sample.Stacks[self.poslist[pos+1][0]].Layers), temp)
                        self.names.insert(pos+1, temps)
                    else:  # Moving inside a stack
                        self.sample.Stacks[self.poslist[pos][0]].Layers.insert(self.poslist[pos][1]-1, temp)  # -2
                        self.names.insert(pos+1, temps)
                    return self.getStringList()
        else:
            return None

class SampleBuilder:
    defs=['Instrument', 'Sample']

    def __init__(self, model):
        self._model_object=model

    def GetModel(self):
        return self._model_object

    def SetModelScript(self, script):
        '''SetModelScript(self, script) --> None

        Sets the script of the current model. This overwrite the current
        script.
        '''
        self._model_object.set_script(script)

    def CompileScript(self):
        '''CompileScript(self) --> None

        Compiles the model script
        '''
        self._model_object.compile_script()

    def GetNewModelScript(self, modelname='models.spec_nx', nb_data_sets=1):
        script='from numpy import *\n'
        script+='import %s as model\n'%modelname
        script+='from models.utils import UserVars, fp, fw, bc, bw\n\n'
        for item in self.defs:
            script+='# BEGIN %s DO NOT CHANGE\n'%item
            script+='# END %s\n\n'%item
        script+='# BEGIN Parameters DO NOT CHANGE\n'
        script+='cp = UserVars()\n'
        script+='# END Parameters\n\n'
        script+='SLD = []\n'
        script+='def Sim(data):\n'
        script+='    I = []\n'
        script+='    SLD[:] = []\n'
        for i in range(nb_data_sets):
            script+='    # BEGIN Dataset %i DO NOT CHANGE\n'%i
            script+='    d = data[%i]\n'%i
            script+='    I.append(sample.SimSpecular(d.x, inst))\n'
            script+='    if _sim: SLD.append(sample.SimSLD(None, None, inst))\n'
            script+='    # END Dataset %i\n'%i
        script+='    return I\n'
        return script

    def BuildNewModel(self, script):
        self.sim_returns_sld=True
        self.SetModelScript(script)
        self.CompileScript()
        self.model=self.GetModel().script_module.model
        names=['Amb', 'Sub']
        Amb=self.model.Layer()
        Sub=self.model.Layer()
        sample=self.model.Sample(Stacks=[], Ambient=Amb, Substrate=Sub)
        # self.sample_widget.SetSample(sample, names)
        self.sampleh=SampleHandler(sample, names)
        self.sampleh.model=self.model

    def insert_code_segment(self, code, descriptor, insert_code):
        '''insert_code_segment(self, code, descriptor, insert_code) --> None

        Inserts code segment into the file. See find_code segment.
        '''
        found=0
        script_lines=code.splitlines(True)
        start_index=-1
        stop_index=-1
        for line in range(len(script_lines)):
            if script_lines[line].find('# BEGIN %s'%descriptor)!=-1:
                start_index=line+1
            if script_lines[line].find('# END %s'%descriptor)!=-1:
                stop_index=line-1
                break

        # Check so everything have preceeded well
        if stop_index<0 and start_index<0:
            raise LookupError('Code segement: %s could not be found'%descriptor)

        # Find the tablevel
        # tablevel = len([' ' for char in script_lines[stop_index+1]\
        #    if char == ' '])
        tablevel=len(script_lines[stop_index+1]) \
                 -len(script_lines[stop_index+1].lstrip())

        # Make the new code tabbed
        tabbed_code=[' '*tablevel+line for line in \
                     insert_code.splitlines(True)]
        # Replace the new code segment with the new
        new_code=''.join(script_lines[:start_index]+tabbed_code \
                         +script_lines[stop_index+1:])

        return new_code

    def find_code_segment(self, code, descriptor):
        '''find_code_segment(self, code, descriptor) --> string

        Finds a segment of code between BEGIN descriptor and END descriptor
        returns a LookupError if the segement can not be found
        '''

        return find_code_segment(code, descriptor)

    def insert_new_data_segment(self, number):
        '''insert_new_data_segment(self, number) --> None

        Inserts a new data segment into the script
        '''
        code=self.GetModel().get_script()
        script_lines=code.splitlines(True)
        line_index=0
        found=0
        for line in script_lines[line_index:]:
            line_index+=1
            if line.find('    return I')!=-1:
                found=1
                break

        if found<1:
            raise LookupError('Could not find "return I" in the script')

        self.AppendSim('Specular', 'inst', ['d.x'])

        script=''.join(script_lines[:line_index-1])
        script+='    # BEGIN Dataset %i DO NOT CHANGE\n'%number
        script+='    d = data[%i]\n'%number
        script+='    I.append(sample.SimSpecular(d.x, inst))\n'
        script+='    if _sim: SLD.append(sample.SimSLD(None, None, inst))\n'
        script+='    # END Dataset %i\n'%number
        script+=''.join(script_lines[line_index-1:])
        self.SetModelScript(script)

    def remove_data_segment(self, number):
        '''remove_data_segment(self, number) --> None

        Removes data segment number
        '''
        code=self.GetModel().get_script()
        found=0
        script_lines=code.splitlines(True)
        start_index=-1
        stop_index=-1
        for line in range(len(script_lines)):
            if script_lines[line].find('# BEGIN Dataset %i'%number)!=-1:
                start_index=line+1
            if script_lines[line].find('# END Dataset %i'%number)!=-1:
                stop_index=line-1
                break

        # Check so everything have preceeded well
        if stop_index<0 and start_index<0:
            raise LookupError('Code segement: %s could not be found'%number)

        script=''.join(script_lines[:start_index-1])
        script+=''.join(script_lines[stop_index+2:])
        self.SetModelScript(script)

    def write_model_script(self, sim_funcs, sim_insts, sim_args,
                           expression_list, parameter_list, instruments):
        script=self.GetModel().get_script()
        # Instrument script creation
        code='from models.utils import create_fp, create_fw\n'
        for inst_name in instruments:
            code+=('%s = model.'%inst_name+
                   instruments[inst_name].__repr__()+'\n')
            code+='%s_fp = create_fp(%s.wavelength);'%(inst_name, inst_name)
            code+=' %s_fw = create_fw(%s.wavelength)\n\n'%(inst_name, inst_name)
        code+=('fp.set_wavelength(inst.wavelength); '
               +'fw.set_wavelength(inst.wavelength)\n')
        script=self.insert_code_segment(script, 'Instrument', code)
        # Sample script creation
        layer_code, stack_code, sample_code=self.sampleh.getCode()
        code=layer_code+'\n'+stack_code+'\n'+sample_code
        script=self.insert_code_segment(script, 'Sample', code)
        # User Vars (Parameters) script creation
        code='cp = UserVars()\n'
        code+=''.join([line+'\n' for line in parameter_list])
        script=self.insert_code_segment(script, 'Parameters', code)
        # Expressions evaluted during simulations (parameter couplings) script creation
        for (i, exps) in enumerate(expression_list):
            exp=[ex+'\n' for ex in exps]
            exp.append('d = data[%i]\n'%i)
            str_arg=', '.join(sim_args[i])
            exp.append('I.append(sample.'
                       'Sim%s(%s, %s))\n'%(sim_funcs[i], str_arg,
                                           sim_insts[i]))
            if self.sim_returns_sld:
                exp.append('if _sim: SLD.append(sample.'
                           'SimSLD(None, None, %s))\n'%sim_insts[i])
            code=''.join(exp)
            script=self.insert_code_segment(script, 'Dataset %i'%i, code)
        self.SetModelScript(script)

    def find_user_parameters(self):
        script=self.GetModel().script
        # Load the custom parameters:
        code=self.find_code_segment(script, 'Parameters')
        uservars_lines=code.splitlines()[1:]
        return uservars_lines

    def find_sim_function_parameters(self):
        # Load the simulation parameters
        script=self.GetModel().script
        sim_exp=[]
        data=self.GetModel().get_data()
        data_names=[di.name for di in data]
        # Lists holding the simulation function arguments
        sim_funcs=[]
        sim_args=[]
        insts=[]
        for i in range(len(data)):
            code=self.find_code_segment(script, 'Dataset %i'%i)
            sim_exp.append([])
            # for line in code.splitlines()[:-1]:
            #    sim_exp[-1].append(line.strip())
            for line in code.splitlines():
                if (line.find('I.append')==-1 and line.find('SLD.append')==-1
                        and line.find('d = data')==-1):
                    # The current line is a command for a parameter
                    sim_exp[-1].append(line.strip())
                elif line.find('I.append')>-1:
                    # The current line is a simulations
                    (tmp, sim_func, args)=line.split('(', 2)
                    sim_funcs.append(sim_func[10:])
                    sim_args.append([arg.strip() for arg in args.split(',')[:-1]])
                    insts.append(args.split(',')[-1][:-2].strip())
        return data_names, insts, sim_args, sim_exp, sim_funcs

    def find_layers_stacks(self, sample_text):
        re_layer=re.compile('([A-Za-z]\w*)\s*=\s*model\.Layer\s*\((.*)\)\n')
        re_stack=re.compile('([A-Za-z]\w*)\s*=\s*model\.Stack\s*\(\s*Layers=\[(.*)\].*\n')
        layers=re_layer.findall(sample_text)
        layer_names=[t[0] for t in layers]
        stacks=re_stack.findall(sample_text)

        all_names=[layer_names.pop(0)]
        for stack in stacks:
            all_names.append(stack[0])
            first_name=stack[1].split(',')[0].strip()
            # check so stack is non-empty
            if first_name!='':
                # Find all items above the first name in the stack
                while (layer_names[0]!=first_name):
                    all_names.append(layer_names.pop(0))
                all_names.append(layer_names.pop(0))
        all_names+=layer_names

        return all_names, layers, stacks

    def find_sample_section(self):
        # Get the current script and split the lines into list items
        script_lines=self.GetModel().get_script().splitlines(True)
        # Try to find out if the script works with multiple SLDs
        for line in script_lines:
            if line.find('SLD[:]')!=-1:
                self.sim_returns_sld=True
                break
            else:
                self.sim_returns_sld=False
        script=''
        # Locate the Sample definition
        line_index=0
        # Start by finding the right section
        found=0
        for line in script_lines[line_index:]:
            line_index+=1
            if line.find('# BEGIN Sample')!=-1:
                found+=1
                break
        sample_text=''
        for line in script_lines[line_index:]:
            line_index+=1
            sample_text+=line
            if line.find('# END Sample')!=-1:
                found+=1
                break
        if found!=2:
            return None
        else:
            return sample_text

    def find_instrument_names(self):
        script=self.GetModel().script
        code=self.find_code_segment(script, 'Instrument')
        re_layer=re.compile('([A-Za-z]\w*)\s*=\s*model\.Instrument\s*\((.*)\)\n')
        instrument_strings=re_layer.findall(code)
        instrument_names=[t[0] for t in instrument_strings]
        return instrument_names

def find_code_segment(code, descriptor):
    '''find_code_segment(code, descriptor) --> string

    Finds a segment of code between BEGIN descriptor and END descriptor
    returns a LookupError if the segement can not be found
    '''
    found=0
    script_lines=code.splitlines(True)
    line_index=0
    for line in script_lines[line_index:]:
        line_index+=1
        if line.find('# BEGIN %s'%descriptor)!=-1:
            found+=1
            break

    text=''
    for line in script_lines[line_index:]:
        line_index+=1
        if line.find('# END %s'%descriptor)!=-1:
            found+=1
            break
        text+=line

    if found!=2:
        raise LookupError('Code segement: %s could not be found'%descriptor)

    return text



'''
====================================
:mod:`sls_sxrd` SLD sxrd data loader
====================================

Loads the data from whitespace seperated column formatted ascii data files.
It is intended for surface x-ray diffraction data where the data sets consists
of rod scans along the l-direction (perpendicular to the surface). The plugin
sorts each rod with equal h and k values into one data sets. The l-direction 
is also sorted.

The default columns are the following:

* First column h values
* Second column k values
* Third values l values
* Fourth column Intensites
* Fifth column The standard deviation of the intensities.

The other settings are just as in the default data loader.

The h,k values is stored as extra data in data.extra_data dictonary as
h and k. 
'''

import numpy as np

from ..data_loader_framework import Template
from ..utils import ShowWarningDialog

try:
    import wx
    from wx.lib.masked import NumCtrl
except ImportError:
    class void():
        pass

    wx=void()
    wx.Dialog=void

class Plugin(Template):
    def __init__(self, parent):
        Template.__init__(self, parent)
        self.h_col=0
        self.k_col=1
        self.l_col=2
        self.I_col=3
        self.eI_col=4
        self.comment='#'
        self.skip_rows=0
        self.delimiter=None

    def LoadData(self, dataset, filename):
        '''LoadData(self, dataset, filename) --> none
        
        Loads the data from filename into the dataset.
        '''
        try:
            load_array=np.loadtxt(filename, delimiter=self.delimiter,
                                  comments=self.comment, skiprows=self.skip_rows)
        except Exception as e:
            ShowWarningDialog(self.parent, 'Could not load the file: '+ \
                              filename+' \nPlease check the format.\n\n numpy.loadtxt' \
                              +' gave the following error:\n'+str(e))
        else:
            # For the freak case of only one data point
            if len(load_array.shape)<2:
                load_array=np.array([load_array])
            # Check so we have enough columns
            if load_array.shape[1]-1<max(self.h_col, self.k_col, self.l_col,
                                         self.I_col, self.eI_col):
                ShowWarningDialog(self.parent, 'The data file does not contain' \
                                  +'enough number of columns. It has '+str(load_array.shape[1]) \
                                  +' columns. Rember that the column index start at zero!')
                # Okay now we have showed a dialog lets bail out ...
                return
            # The data is set by the default Template.__init__ function, neat hu
            # Note that the loaded data goes into *_raw so that they are not
            # changed by the transforms

            # Create an record array so we can sort the data properly
            data=np.rec.fromarrays([
                load_array[:, self.h_col].round().astype(type(1)),
                load_array[:, self.k_col].round().astype(type(1)),
                load_array[:, self.l_col], load_array[:, self.I_col],
                load_array[:, self.eI_col]
                ],
                names='h, k, l, I, eI')
            # Sort the data
            data.sort(order=('h', 'k', 'l'))
            i=0
            while i<len(data):
                # Find all the data for each rod
                tmp=data.compress(np.bitwise_and(data['h']==data[i]['h'],
                                                 data['k']==data[i]['k']))
                self.data.add_new('(%i, %i)'%(tmp['h'][0], tmp['k'][0]))
                self.data[-1].x_raw=tmp['l']
                self.data[-1].y_raw=tmp['I']
                self.data[-1].error_raw=tmp['eI']
                # Run the commands on the data - this also sets the x,y, error memebers
                # of that data item.
                self.data[-1].run_command()
                self.data[-1].set_extra_data('h', tmp['h'], 'h')
                self.data[-1].set_extra_data('k', tmp['k'], 'k')
                # Increase the index
                i+=len(tmp)

            # Update the data list
            self.UpdateDataList()
            # Send an update that new data has been loaded
            self.SendUpdateDataEvent()

    def SettingsDialog(self):
        '''SettingsDialog(self) --> None
        
        This function should - if necessary implement a dialog box
        that allows the user set import settings for example.
        '''
        col_values={'I': self.I_col, 'h': self.h_col, 'k': self.k_col,
                    'l': self.l_col, 'I error': self.eI_col}
        misc_values={'Comment': str(self.comment), 'Skip rows': self.skip_rows,
                     'Delimiter': str(self.delimiter)}
        dlg=SettingsDialog(self.parent, col_values, misc_values)
        if dlg.ShowModal()==wx.ID_OK:
            col_values=dlg.GetColumnValues()
            misc_values=dlg.GetMiscValues()
            self.h_col=col_values['h']
            self.k_col=col_values['k']
            self.l_col=col_values['l']
            self.I_col=col_values['I']
            self.eI_col=col_values['I error']
            self.comment=misc_values['Comment']
            self.skip_rows=misc_values['Skip rows']
            self.delimiter=misc_values['Delimiter']
            self.SetStatusText('Changed import settings')
        else:
            self.SetStatusText('No change to import settings')
        dlg.Destroy()

class SettingsDialog(wx.Dialog):

    def __init__(self, parent, col_values, misc_values):
        wx.Dialog.__init__(self, parent, -1, 'Data loader settings')

        box_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Make the box for putting in the columns
        col_box=wx.StaticBox(self, -1, "Columns")
        col_box_sizer=wx.StaticBoxSizer(col_box, wx.VERTICAL)

        # col_values = {'y': 1,'x': 0,'y error': 1}
        col_grid=wx.GridBagSizer(len(col_values), 2)
        self.col_controls=col_values.copy()
        keys=list(col_values.keys())
        keys.sort()
        for i, name in enumerate(keys):
            text=wx.StaticText(self, -1, name+': ')
            control=wx.SpinCtrl(self)
            control.SetRange(0, 100)
            control.SetValue(col_values[name])
            col_grid.Add(text, (i, 0),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            col_grid.Add(control, (i, 1),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            self.col_controls[name]=control

        col_box_sizer.Add(col_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        box_sizer.Add(col_box_sizer, 0, wx.ALL | wx.EXPAND, 5)

        col_box=wx.StaticBox(self, -1, "Misc")
        col_box_sizer=wx.StaticBoxSizer(col_box, wx.VERTICAL)

        # Lets add another box for comments and rows to skip
        # misc_values = {'Comment': '#', 'Skip rows': 0,'Delimiter': 'None'}
        col_grid=wx.GridBagSizer(len(misc_values), 2)
        self.misc_controls=misc_values.copy()
        keys=list(misc_values.keys())
        keys.sort()
        for i, name in enumerate(keys):
            text=wx.StaticText(self, -1, name+': ')
            if type(misc_values[name])==type(1):
                control=wx.SpinCtrl(self)
                control.SetRange(0, 100)
                control.SetValue(misc_values[name])
            else:
                control=wx.TextCtrl(self, value=misc_values[name],
                                    style=wx.EXPAND)
            col_grid.Add(text, (i, 0),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            col_grid.Add(control, (i, 1),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            self.misc_controls[name]=control

        col_box_sizer.Add(col_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        box_sizer.Add(col_box_sizer, 0, wx.ALL | wx.EXPAND, 5)

        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        button_sizer.Realize()

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(box_sizer, 1, wx.GROW, 20)
        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW, 30)

        sizer.Add(button_sizer, 0,
                  flag=wx.ALIGN_RIGHT, border=20)
        self.SetSizer(sizer)

        sizer.Fit(self)
        self.Layout()

    def GetColumnValues(self):
        values={}
        for key in self.col_controls:
            values[key]=self.col_controls[key].GetValue()
        return values

    def GetMiscValues(self):
        values={}
        for key in self.misc_controls:
            val=self.misc_controls[key].GetValue()
            if type(val)==type('') or type(val)==type(''):
                if val.lower()=='none':
                    val=None
            values[key]=val
        return values



''' Library for reflectivity calculations with neutrons.
Programmed by Matts Bjorck
Last changed 2017-09-02
'''
from numpy import *
from . import math_utils as mu
from . import int_lay_xmean
from functools import reduce
from genx.gui_logging import iprint

def make_D(q_p, q_m):
    return mat([[1, 1, 0, 0], [q_p, -q_p, 0, 0], [0, 0, 1, 1], [0, 0, q_m, -q_m]])

def make_P(q_p, q_m, d):
    return mat([[exp(-1.0j*q_p*d), 0, 0, 0], [0, exp(1.0j*q_p*d), 0, 0],
                [0, 0, exp(-1.0j*q_m*d), 0], [0, 0, 0, exp(1.0j*q_m*d)]])

def make_R(theta_diff):
    ct=cos(theta_diff/2.0)
    st=sin(theta_diff/2.0)
    return mat([[ct, 0, st, 0], [0, ct, 0, st], [-st, 0, ct, 0], [0, -st, 0, ct]])

def make_Sigma(q_p, q_m, sigma):
    ep=exp(-q_p**2*sigma**2)
    em=exp(-q_m**2*sigma**2)
    return mat([[ep, 0, 0, 0], [0, 1, 0, 0], [0, 0, em, 0], [0, 0, 0, 1]])

def ReflOld(Q, Vp, Vm, d, M_ang):
    ''' Calculates spin-polarized reflectivity according to S.J. Blundell 
        and J.A.C. Bland Phys rev. B. vol 46 3391 (1992)
        Input parameters:   Q : Scattering vector in reciprocal 
                                angstroms Q=4*pi/lambda *sin(theta)
                            Vp: Neutron potential for spin up
                            Vm: Neutron potential for spin down
                            d: layer thickness
                            M_ang: Angle of the magnetic 
                                    moment(radians!) M_ang=0 =>M//nuetron spin
        No roughness is included!
        Returns:            (Ruu,Rdd,Rud,Rdu)
                            (up-up,down-down,up-down,down-up)
    '''
    # Assume first element=substrate and last=ambient!
    Q=Q/2.0
    # Wavevectors in the layers
    Qi_p=sqrt(Q[:, newaxis]**2-Vp)
    Qi_m=sqrt(Q[:, newaxis]**2-Vm)
    # print Qi_p.shape
    # print d.shape
    # print M_ang.shape
    # M_ang=M_ang[::-1]
    # d=d[::-1]
    # Angular difference between the magnetization
    theta_diff=M_ang[1:]-M_ang[:-1]  # Unsure but think this is correct

    # theta_diff=theta_diff[::-1]
    # print theta_diff
    def calc_mat(q_p, q_m, d, theta_diff):
        return make_D(q_p, q_m)*make_P(q_p, q_m, d)*make_D(q_p, q_m)**-1*make_R(theta_diff)

    # Calculate the transfer matrix - this implementation is probably REALLY slow...
    M=[make_D(qi_p[-1], qi_m[-1])**-1*make_R(theta_diff[-1])*reduce(lambda x, y: y*x,
                                                                    [calc_mat(q_p, q_m, di, theta_diffi)
                                                                     for q_p, q_m, di, theta_diffi in
                                                                     zip(qi_p[1:-1], qi_m[1:-1], d[1:-1],
                                                                         theta_diff[:-1])]
                                                                    , make_D(qi_p[0], qi_m[0]))
       for qi_p, qi_m in zip(Qi_p, Qi_m)]
    # M=[make_D(qi_p[0],qi_m[0])**-1*make_R(theta_diff[0])*reduce(lambda x,y:y*x,[calc_mat(q_p,q_m,di,theta_diffi)
    #     for q_p,q_m,di,theta_diffi in zip(qi_p[1:-1],qi_m[1:-1],d[1:-1],theta_diff[1:])]
    #       ,make_D(qi_p[-1],qi_m[-1]))
    #       for qi_p,qi_m in zip(Qi_p,Qi_m)]
    # print 'Matrix calculated'
    # print M[0]
    # transform M into an array - for fast indexing
    M=array([array(m) for m in M])
    # print M.shape
    Ruu=(M[:, 1, 0]*M[:, 2, 2]-M[:, 1, 2]*M[:, 2, 0])/(M[:, 0, 0]*M[:, 2, 2]-M[:, 0, 2]*M[:, 2, 0])
    Rud=(M[:, 3, 0]*M[:, 2, 2]-M[:, 3, 2]*M[:, 2, 0])/(M[:, 0, 0]*M[:, 2, 2]-M[:, 0, 2]*M[:, 2, 0])
    Rdu=(M[:, 1, 2]*M[:, 0, 0]-M[:, 1, 0]*M[:, 0, 2])/(M[:, 0, 0]*M[:, 2, 2]-M[:, 0, 2]*M[:, 2, 0])
    Rdd=(M[:, 3, 2]*M[:, 0, 0]-M[:, 3, 0]*M[:, 0, 2])/(M[:, 0, 0]*M[:, 2, 2]-M[:, 0, 2]*M[:, 2, 0])
    # print 'Reflectivites calculated'
    return abs(Ruu)**2, abs(Rdd)**2, abs(Rud)**2, abs(Rdu)**2

# ======================================================================
# New quicker way of doing spin pol. calcs.

ctype=complex128

def ass_X_2int(k_mj, k_mj1, k_pj, k_pj1, theta_diff):
    costd=cos(theta_diff/2.0)
    sintd=sin(theta_diff/2.0)
    X=zeros((4, 4)+k_pj.shape, dtype=ctype)
    X[0, 0]=costd*(k_pj1+k_pj)/2./k_pj
    X[0, 1]=-costd*(k_pj1-k_pj)/2./k_pj
    X[0, 2]=sintd*(k_pj+k_mj1)/2./k_pj
    X[0, 3]=sintd*(k_pj-k_mj1)/2./k_pj
    # X[0] = X[0]/2/k_pj
    X[1, 0]=X[0, 1]  # -(costd*(k_pj1 - k_pj))/(2*k_pj)
    X[1, 1]=X[0, 0]  # (costd*(k_pj1 + k_pj))/(2*k_pj)
    X[1, 2]=X[0, 3]  # (sintd*(k_pj - k_mj1))/(2*k_pj)
    X[1, 3]=X[0, 2]  # (sintd*(k_pj + k_mj1))/(2*k_pj)
    X[2, 0]=-(sintd*(k_pj1+k_mj))/(2.*k_mj)
    X[2, 1]=(sintd*(k_pj1-k_mj))/(2.*k_mj)
    X[2, 2]=(costd*(k_mj1+k_mj))/(2.*k_mj)
    X[2, 3]=-(costd*(k_mj1-k_mj))/(2.*k_mj)
    X[3, 0]=X[2, 1]  # (sintd*(k_pj1 - k_mj))/(2*k_mj)
    X[3, 1]=X[2, 0]  # -(sintd*(k_pj1 + k_mj))/(2*k_mj)
    X[3, 2]=X[2, 3]  # -(costd*(k_mj1 - k_mj))/(2*k_mj)
    X[3, 3]=X[2, 2]  # (costd*(k_mj1 + k_mj))/(2*k_mj)
    return X

def ass_X(k_p, k_m, theta_diff):
    ''' Make the interface transmission matrix for neutron reflection from 
    a interface.
    '''
    # First is the substrate and last is the ambient (j=0)
    # The order is then [N .. j+1, j, ....0]
    k_pj1=k_p[:, :-1]
    k_pj=k_p[:, 1:]
    k_mj1=k_m[:, :-1]
    k_mj=k_m[:, 1:]
    return ass_X_2int(k_mj, k_mj1, k_pj, k_pj1, theta_diff)

def gauss(q, sigma2):
    '''Fourier transform of the interface roughness weight function
    '''
    return exp(-q**2*sigma2/2.0)

def include_sigma(X, k_p, k_m, sigma, w=gauss):
    '''Function to include roughness into the interface matrix.
    '''
    sigma2=sigma[..., :-1]**2
    k_pj1=k_p[:, :-1]
    k_pj=k_p[:, 1:]
    k_mj1=k_m[:, :-1]
    k_mj=k_m[:, 1:]
    X[0, 0]=X[0, 0]*w(k_pj-k_pj1, sigma2)
    X[0, 1]=X[0, 1]*w(k_pj+k_pj1, sigma2)
    X[0, 2]=X[0, 2]*w(k_pj-k_mj1, sigma2)
    X[0, 3]=X[0, 3]*w(k_pj+k_mj1, sigma2)
    X[1, 0]=X[0, 1]  # X[1,0]*w(k_pj + k_pj1, sigma2)
    X[1, 1]=X[0, 0]  # X[1,1]*w(k_pj - k_pj1, sigma2)
    X[1, 2]=X[0, 3]  # X[1,2]*w(k_pj + k_mj1, sigma2)
    X[1, 3]=X[0, 2]  # X[1,3]*w(k_pj - k_mj1, sigma2)
    X[2, 0]=X[2, 0]*w(k_mj-k_pj1, sigma2)
    X[2, 1]=X[2, 1]*w(k_mj+k_pj1, sigma2)
    X[2, 2]=X[2, 2]*w(k_mj-k_mj1, sigma2)
    X[2, 3]=X[2, 3]*w(k_mj+k_mj1, sigma2)
    X[3, 0]=X[2, 1]  # X[3,0]*w(k_mj + k_pj1, sigma)
    X[3, 1]=X[2, 0]  # X[3,1]*w(k_mj - k_pj1, sigma)
    X[3, 2]=X[2, 3]  # X[3,2]*w(k_mj + k_mj1, sigma)
    X[3, 3]=X[2, 2]  # X[3,3]*w(k_mj - k_mj1, sigma)
    return X

def ass_P(k_p, k_m, d):
    ''' Make the layer proagation matrix for a layer.
    '''
    P=zeros((4, 4)+k_p.shape, dtype=ctype)
    P[0, 0]=exp(-1.0J*k_p*d)
    P[1, 1]=1/P[0, 0]  # exp(1.0J*k_p*d)
    P[2, 2]=exp(-1.0J*k_m*d)
    P[3, 3]=1/P[2, 2]  # exp(1.0J*k_m*d)
    return P

def Refl(Q, Vp, Vm, d, M_ang, sigma=None, return_int=True):
    '''A quicker implementation than the ordinary slow implementaion in Refl
    Calculates spin-polarized reflectivity according to S.J. Blundell 
        and J.A.C. Bland Phys rev. B. vol 46 3391 (1992)
        The algorithm assumes that the first element in the arrays represents
        the substrate and the last the ambient layer.
        Input parameters:   Q : Scattering vector in reciprocal 
                                angstroms Q=4*pi/lambda *sin(theta)
                            Vp: Neutron potential for spin up
                            Vm: Neutron potential for spin down
                            d: layer thickness
                            M_ang: Angle of the magnetic 
                                    moment(radians!) M_ang=0 =>M//neutron spin
                            sigma: The roughness of the upper interface.
                            return_int: Flag for returning the instensity, default=True. If False return the amplitudes.
        Returns:            (Ruu,Rdd,Rud,Rdu)
                            (up-up,down-down,up-down,down-up)
    '''
    # Assume first element=substrate and last=ambient!
    k_amb=Q[:, newaxis]/2.0
    # Wavevectors in the layers
    k_p=sqrt(k_amb**2-Vp.astype(complex128))
    k_m=sqrt(k_amb**2-Vm.astype(complex128))
    # Angular difference between the magnetization
    theta_diff=M_ang[1:]-M_ang[:-1]
    # if sigma is None:
    #    sigma = zeros(d.shape)
    # Assemble the interface reflectivity matrix
    X=ass_X(k_p, k_m, theta_diff)
    if sigma is not None:
        X=include_sigma(X, k_p, k_m, sigma)
    # Assemble the layer propagation matrices
    P=ass_P(k_p, k_m, d)
    # Multiply the propagation matrices with the interface matrix
    PX=mu.dot4_Adiag(P[..., 1:-1], X[..., :-1])
    # Multiply up the sample matrix
    # print 'X: ', X[:,:, 0, -1]
    M=mu.dot4(X[..., -1], reduce(mu.dot4, rollaxis(PX, 3)[::-1]))
    # print M.shape
    # print 'M: ', M[:,:, 0]
    # print 'denom: ',  M[0,0]*M[2,2]-M[0,2]*M[2,0]
    denom=M[0, 0]*M[2, 2]-M[0, 2]*M[2, 0]
    Ruu=(M[1, 0]*M[2, 2]-M[1, 2]*M[2, 0])/denom
    Rud=(M[3, 0]*M[2, 2]-M[3, 2]*M[2, 0])/denom
    Rdu=(M[1, 2]*M[0, 0]-M[1, 0]*M[0, 2])/denom
    Rdd=(M[3, 2]*M[0, 0]-M[3, 0]*M[0, 2])/denom

    if return_int:
        return abs(Ruu)**2, abs(Rdd)**2, abs(Rud)**2, abs(Rdu)**2
    else:
        return Ruu, Rdd, Rud, Rdu

def Refl_int_lay(Q, V0, Vmag, d, M_ang, sigma, dmag_u, dd_u, M_ang_u, sigma_u,
                 dmag_l, dd_l, M_ang_l, sigma_l, return_int=True):
    '''A quicker implementation than the ordinary slow implementaion in Refl
    Calculates spin-polarized reflectivity according to S.J. Blundell
        and J.A.C. Bland Phys rev. B. vol 46 3391 (1992)
        The algorithm assumes that the last element in the arrays represents
        the substrate and the first the ambient layer.
        Input parameters:   Q : Scattering vector in reciprocal
                                angstroms Q=4*pi/lambda *sin(theta)
                            Vp: Neutron potential for spin up
                            Vm: Neutron potential for spin down
                            d: layer thickness
                            M_ang: Angle of the magnetic
                                    moment(radians!) M_ang=0 =>M//nuetron spin
                            sigma: The roughness of the upper interface.
                            The subscript l and u denotes the lower and upper interface
                            , respectively.
        Returns:            (Ruu,Rdd,Rud,Rdu)
                            (up-up,down-down,up-down,down-up)
    '''
    Vp=V0+Vmag
    Vm=V0-Vmag
    Vp_u=V0+Vmag*(1.+dmag_u)
    Vm_u=V0-Vmag*(1.+dmag_u)
    Vp_l=V0+Vmag*(1.+dmag_l)
    Vm_l=V0-Vmag*(1.+dmag_l)
    # Assume last element=substrate and first=ambient!
    k_amb=Q[:, newaxis]/2.0
    # Wavevectors in the layers
    kp=sqrt(k_amb**2-Vp).astype(complex128)
    km=sqrt(k_amb**2-Vm).astype(complex128)
    kp_u=sqrt(k_amb**2-Vp_u).astype(complex128)
    km_u=sqrt(k_amb**2-Vm_u).astype(complex128)
    kp_l=sqrt(k_amb**2-Vp_l).astype(complex128)
    km_l=sqrt(k_amb**2-Vm_l).astype(complex128)
    # Angular difference between the magnetization
    theta_diff=M_ang[:-1]-M_ang[1:]
    theta_diff_lu=M_ang_l[:-1]-M_ang_u[1:]
    theta_diff_l=M_ang[:-1]-M_ang_l[:-1]
    theta_diff_u=M_ang_u[1:]-M_ang[1:]

    X_lu=ass_X_2int(km_l[:, :-1], km_u[:, 1:], kp_l[:, :-1], kp_u[:, 1:], theta_diff_lu)
    X_l=ass_X_2int(km[:, :-1], km_l[:, :-1], kp[:, :-1], kp_l[:, :-1], theta_diff_l)
    X_u=ass_X_2int(km_u[:, 1:], km[:, 1:], kp_u[:, 1:], kp[:, 1:], theta_diff_u)

    X=int_lay_xmean.calc_neu_Xmean(X_l, X_lu, X_u, km, kp, km_l, kp_l, km_u, kp_u, dd_u, dd_l, sigma, sigma_l, sigma_u)

    # Assemble the layer propagation matrices
    P=ass_P(kp, km, d-dd_u-dd_l)
    # Multiply the propagation matrices with the interface matrix
    PX=mu.dot4_Adiag(P[..., 1:-1], X[..., 1:])
    # Multiply up the sample matrix
    M=mu.dot4(X[..., 0], reduce(mu.dot4, rollaxis(PX, 3)))
    # print M.shape
    denom=M[0, 0]*M[2, 2]-M[0, 2]*M[2, 0]
    Ruu=(M[1, 0]*M[2, 2]-M[1, 2]*M[2, 0])/denom
    Rud=(M[3, 0]*M[2, 2]-M[3, 2]*M[2, 0])/denom
    Rdu=(M[1, 2]*M[0, 0]-M[1, 0]*M[0, 2])/denom
    Rdd=(M[3, 2]*M[0, 0]-M[3, 0]*M[0, 2])/denom

    if return_int:
        return abs(Ruu)**2, abs(Rdd)**2, abs(Rud)**2, abs(Rdu)**2
    else:
        return Ruu, Rdd, Rud, Rdu

from . import USE_NUMBA

if USE_NUMBA:
    # try to use numba to speed up the calculation intensive functions:
    try:
        from .neutron_numba import Refl
    except Exception as e:
        iprint('Could not use numba, no speed up from JIT compiler:\n'+str(e))

if __name__=='__main__':
    Q=arange(0.01, 0.2, 0.0005)
    sld_Fe=8e-6
    sld_Fe_p=12.9e-6
    sld_Fe_m=2.9e-6
    sld_Pt=6.22e-6

    def pot(sld):
        lamda=5.0
        # return (2*pi/lamda)**2*(1-(1-lamda**2/2/pi*sld)**2)
        return sld/pi

    Vp=array([pot(sld_Pt), pot(sld_Fe_p), pot(sld_Pt), pot(sld_Fe_p), 0])
    Vm=array([pot(sld_Pt), pot(sld_Fe_m), pot(sld_Pt), pot(sld_Fe_m), 0])
    d=array([3, 100, 50, 100, 3])
    M_ang=array([0.0, 45*pi/180, 0.0, 90*pi/180, 0.0, ])
    sigma=array([10., 10., 10., 10., 10.0])*0
    import time

    t1=time.time()
    for i in range(10):
        r=Refl(Q, Vp, Vm, d, M_ang, sigma)
    t2=time.time()
    for i in range(10):
        r_orig=ReflOld(Q, Vp, Vm, d, M_ang)
    t3=time.time()
    iprint('Old version: ', t3-t2)
    iprint('New version: ', t2-t1)
    iprint('Speedup: ', (t3-t2)/(t2-t1))
    from pylab import *

    # plot(Q,log10(r[0]+1e-6),Q,log10(r[1]+1e-6),Q,log10(r[2]+1e-6),Q,log10(r[3]+1e-6))
    # io.write_array(open('test.dat','w'),array(zip(Q,abs(r[0]),abs(r[1]),abs(r[2]))))
    for rc in r:
        plot(Q, log10(abs(rc)))
    for rc in r_orig:
        plot(Q, log10(abs(rc)), '.')
    iprint('Done')
    show()
    if True:
        import profile

        profile.run('[Refl(Q,Vp,Vm,d,M_ang, sigma) for i in range(50)]')



'''
Just a library of the different eventhandler needed in the GUI
File started by: Matts Bjorck
'''
from . import version

__version__=version.__version__

import _thread, time
import wx, os, traceback
from io import StringIO
import wx.adv
from wx.lib.wordwrap import wordwrap
import webbrowser
import numpy as np
from logging import debug

from . import model as modellib
from . import solvergui, help
from . import filehandling as io
from .gui_logging import iprint

manual_url='https://aglavic.github.io/genx/doc/'
homepage_url='https://aglavic.github.io/genx/'

def get_pages(frame):
    # Get all plot panel objects in GUI
    pages=[]
    for page in frame.plot_notebook.GetChildren():
        pages+=page.GetChildren()
    if frame.sep_plot_notebook is not frame.plot_notebook:
        for page in frame.sep_plot_notebook.GetChildren():
            pages+=page.GetChildren()

    # pages = [frame.plot_data, frame.plot_fom, frame.plot_pars,\
    #             frame.plot_fomscan]
    return pages

def set_title(frame):
    filepath, filename=os.path.split(frame.model.filename)
    if filename!='':
        if frame.model.saved:
            frame.SetTitle(filename+' - '+filepath+' - GenX ' \
                           +__version__)
        else:
            frame.SetTitle(filename+'* - '+filepath+' - GenX ' \
                           +__version__)
    else:
        frame.SetTitle('GenX '+__version__)

def models_changed(frame, event):
    '''models_changed(frame, event) --> None
    
    callback when something has changed in the model so that the 
    user can be made aware that the model needs saving.
    '''
    try:
        frame.model.saved=not event.permanent_change
    except AttributeError:
        frame.model.saved=False
    else:
        frame.plugin_control.OnGridChanged(event)
    set_title(frame)

def new(frame, event):
    '''
    new(frame, event) --> None
    
    Event handler for creating a new model
    '''
    # print "Event handler: new"
    if not frame.model.saved:
        ans=ShowQuestionDialog(frame, 'If you continue any changes in'
                                      ' your model will not be saved.',
                               'Model not saved')
        if not ans:
            return

    # Reset the model - remove everything from the previous model
    frame.model.new_model()
    # Update all components so all the traces are gone.
    _post_new_model_event(frame, frame.model, desc='Fresh model')
    frame.plugin_control.OnNewModel(None)
    frame.main_frame_statusbar.SetStatusText('New model created', 1)
    set_title(frame)
    frame.model.saved=True

def open(frame, event):
    '''
    open(frame, event) --> None
    
    Event handler for opening a model file...
    '''
    # Check so the model is saved before quitting
    if not frame.model.saved:
        ans=ShowQuestionDialog(frame, 'If you continue any changes in'
                                      ' your model will not be saved.',
                               'Model not saved')
        if not ans:
            return

    dlg=wx.FileDialog(frame, message="Open", defaultFile="",
                      wildcard="GenX File (*.hgx;*.gx)|*.hgx;*.gx",
                      style=wx.FD_OPEN  # | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        path=dlg.GetPath()
        debug('open: path retrieved')
        open_model(frame, path)

    dlg.Destroy()

def open_model(frame, path):
    debug('open_model: clear model')
    frame.model.new_model()
    frame.paramter_grid.PrepareNewModel()
    # Update all components so all the traces are gone.
    # _post_new_model_event(frame, frame.model)
    debug('open_model: load_file')
    try:
        io.load_file(path, frame.model, frame.solver_control.optimizer, frame.config)
    except modellib.IOError as e:
        ShowModelErrorDialog(frame, e.__str__())
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        iprint('Error in loading the file ', path, '. Pyton traceback:\n ', val)
        ShowErrorDialog(frame, 'Could not open the file. Python Error:\n%s'%(val,))
        return
    debug('open_model: read config')
    try:
        [p.ReadConfig() for p in get_pages(frame)]
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        iprint('Error in loading config for the plots. Pyton traceback:\n ', val)
        ShowErrorDialog(frame, 'Could not read the config for the plots. Python Error:\n%s'%(val,))
    try:
        frame.paramter_grid.ReadConfig()
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        iprint('Error in loading config for parameter grid. Pyton traceback:\n ', val)
        ShowErrorDialog(frame, 'Could not read the config for the parameter grid. Python Error:\n%s'%(val,))
    else:
        # Update the Menu choice
        frame.main_frame_menubar.mb_view_grid_slider.Check(frame.paramter_grid.GetValueEditorSlider())
    debug('open_model: update plugins')
    try:
        frame.plugin_control.OnOpenModel(None)
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, 'Problems when plugins processed model.' \
                               ' Python Error:\n%s'%(val,))
    frame.main_frame_statusbar.SetStatusText('Model loaded from file',
                                             1)
    # Post an event to update everything else
    debug('open_model: post new model event')
    _post_new_model_event(frame, frame.model)
    # Needs to put it to saved since all the widgets will have 
    # been updated
    frame.model.saved=True
    set_title(frame)

def on_new_model(frame, event):
    '''
    on_new_model(frame, event) --> None
    
    Callback for NEW_MODEL event. Used to update the script for
    a new model i.e. put the string to the correct value. 
    '''
    # print "Callback: on_new_model"
    # Set the string in the script_editor
    frame.script_editor.SetText(event.GetModel().get_script())
    # Let the solvergui do its loading and updating:
    frame.solver_control.ModelLoaded()
    # Lets update the mb_use_toggle_show Menu item
    frame.main_frame_menubar.mb_use_toggle_show.Check(
        frame.config.get_boolean('data handling', 'toggle show'))
    try:
        val=frame.config.get_boolean('parameter grid', 'auto sim')
    except io.OptionError:
        iprint('Could not locate option parameters.auto sim')
        frame.main_frame_menubar.mb_fit_autosim.Check(True)
    else:
        frame.main_frame_menubar.mb_fit_autosim.Check(val)

    # Let other event handlers recieve the event as well
    event.Skip()

def update_for_save(frame):
    """Updates the various objects for a save"""
    frame.model.set_script(frame.script_editor.GetText())
    # Save the current state of autosim to the config
    frame.config.set('parameter grid', 'auto sim', frame.main_frame_menubar.mb_fit_autosim.IsChecked())

def save(frame, event):
    '''
    save(frame, event) --> None
    
    Event handler for saving a model file ...
    '''
    update_for_save(frame)
    fname=frame.model.get_filename()
    # If model hasn't been saved
    if fname=='':
        # Proceed with calling save as
        save_as(frame, event)
    else:
        # If it has been saved just save it
        try:
            io.save_file(fname, frame.model, frame.solver_control.optimizer, frame.config)
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, e.__str__())
        except Exception as e:
            outp=StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, 'Could not save the file. Python Error: \n%s'%(val,))
        set_title(frame)

    frame.main_frame_statusbar.SetStatusText('Model saved to file', 1)

def save_as(frame, event):
    '''save_as(frame, event) --> None
    
    Event handler for save as ...
    '''
    dlg=wx.FileDialog(frame, message="Save As", defaultFile="",
                      wildcard="HDF5 GenX File (*.hgx)|*.hgx|GenX File (*.gx)|*.gx",
                      style=wx.FD_SAVE  # | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        update_for_save(frame)
        fname=dlg.GetPath()
        base, ext=os.path.splitext(fname)
        if ext=='':
            ext='.hgx'
        fname=base+ext
        result=True
        if os.path.exists(fname):
            filepath, filename=os.path.split(fname)
            result=ShowQuestionDialog(frame, 'The file %s already exists. Do you wish to overwrite it?'%filename,
                                      'Overwrite?')
        if result:
            try:
                io.save_file(fname, frame.model, frame.solver_control.optimizer,
                             frame.config)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, e.__str__())
            except Exception as e:
                outp=StringIO()
                traceback.print_exc(200, outp)
                val=outp.getvalue()
                outp.close()
                ShowErrorDialog(frame, 'Could not save the file. Python Error:\n%s'%(val,))
            set_title(frame)

    dlg.Destroy()

def export_orso(frame, event):
    '''export_data(frame, event) --> None

    exports the data to one file per data set with a basename with
    extention given by a save dialog.
    '''
    dlg=wx.FileDialog(frame, message="Export data and model", defaultFile="",
                      wildcard="ORSO Text File (*.ort)|*.ort",
                      style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        try:
            frame.model.export_orso(dlg.GetPath())
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(
                'Error when exporting data', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e), 'export data - model.export_orso')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Data exported', 1)

    dlg.Destroy()

def export_data(frame, event):
    '''export_data(frame, event) --> None
    
    exports the data to one file per data set with a basename with
    extention given by a save dialog.
    '''
    dlg=wx.FileDialog(frame, message="Export data", defaultFile="",
                      wildcard="Dat File (*.dat)|*.dat",
                      style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        try:
            frame.model.export_data(dlg.GetPath())
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(
                'Error when exporting data', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e), 'export data - model.export_data')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Data exported', 1)

    dlg.Destroy()

def export_script(frame, event):
    '''export_script(frame, event) --> None
    
    Exports the script to a python file given by a filedialog.
    '''
    dlg=wx.FileDialog(frame, message="Export data", defaultFile="",
                      wildcard="Python File (*.py)|*.py",
                      style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        fname=dlg.GetPath()
        base, ext=os.path.splitext(fname)
        if ext=='':
            ext='.py'
        fname=base+ext
        result=True
        if os.path.exists(fname):
            filepath, filename=os.path.split(fname)
            result=ShowQuestionDialog(frame,
                                      'The file %s already exists. Do you wish to overwrite it?'%filename
                                      , 'Overwrite?')
        if result:
            try:
                # frame.model.export_script(dlg.GetPath())
                frame.model.export_script(fname)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, str(e))
                frame.main_frame_statusbar.SetStatusText(
                    'Error when exporting script', 1)
                return
            except Exception as e:
                ShowErrorDialog(frame, str(e),
                                'export script - model.export_script')
                frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
                return
            else:
                frame.main_frame_statusbar.SetStatusText(
                    'Script exported to file', 1)

    dlg.Destroy()

def export_table(frame, event):
    '''export_table(frame, event) --> None
    
    Exports the table to a dat file given by a filedialog.
    '''
    dlg=wx.FileDialog(frame, message="Export data", defaultFile="",
                      wildcard="Table File (*.tab)|*.tab",
                      style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        fname=dlg.GetPath()
        base, ext=os.path.splitext(fname)
        if ext=='':
            ext='.tab'
        fname=base+ext
        result=True
        if os.path.exists(fname):
            filepath, filename=os.path.split(fname)
            result=ShowQuestionDialog(frame,
                                      'The file %s already exists. Do you wish to overwrite it?'%filename
                                      , 'Overwrite?')
        if result:
            try:
                # frame.model.export_table(dlg.GetPath())
                frame.model.export_table(fname)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, str(e))
                frame.main_frame_statusbar.SetStatusText(
                    'Error when exporting table', 1)
                return
            except Exception as e:
                ShowErrorDialog(frame, str(e),
                                'export table - model.export_table')
                frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
                return
            else:
                frame.main_frame_statusbar.SetStatusText(
                    'Table exported to file', 1)

    dlg.Destroy()

def import_script(frame, event):
    '''import_script(frame, event) --> None
    
    imports a script from the file given by a file dialog box
    '''
    dlg=wx.FileDialog(frame, message="Import script", defaultFile="",
                      wildcard="Python files (*.py)|*.py|All files (*.*)|*.*",
                      style=wx.FD_OPEN | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        try:
            frame.model.import_script(dlg.GetPath())
            # frame.model.import_script(fname)
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(
                'Error when importing script', 1)
            return
        except Exception as e:
            ShowErrorDialog(frame, str(e),
                            'import script - model.import_script')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
            return
        try:
            frame.plugin_control.OnOpenModel(None)
        except Exception as e:
            outp=StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, 'Problems when plugins processed model.' \
                                   ' Python Error:\n%s'%(val,))
        else:
            frame.main_frame_statusbar.SetStatusText(
                'Script imported from file', 1)
    dlg.Destroy()
    # Post event to tell that the model has changed
    _post_new_model_event(frame, frame.model)

def import_data(frame, event):
    ''' import_data(frame, event) -->None
    
    callback to import data into the program
    '''
    # Reuse of the callback in the datalist.DataController
    try:
        frame.data_list.eh_tb_open(event)
    except Exception as e:
        ShowErrorDialog(frame, str(e),
                        'import data - data_list.eh_tb_open')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        return

def import_table(frame, event):
    '''import_table(frame, event) --> None
    
    imports a table from the file given by a file dialog box
    '''
    dlg=wx.FileDialog(frame, message="Import script", defaultFile="",
                      wildcard="Table File (*.tab)|*.tab|All files (*.*)|*.*",
                      style=wx.FD_OPEN | wx.FD_CHANGE_DIR
                      )
    if dlg.ShowModal()==wx.ID_OK:
        try:
            frame.model.import_table(dlg.GetPath())
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(
                'Error when importing script', 1)
            dlg.Destroy()
            return
        except Exception as e:
            ShowErrorDialog(frame, str(e),
                            'import script - model.import_script')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
            dlg.Destroy()
            return

    dlg.Destroy()
    # Post event to tell that the model has cahnged
    _post_new_model_event(frame, frame.model)
    frame.main_frame_statusbar.SetStatusText('Table imported from file', 1)

def parameter_value_changed(frame, event):
    """ Event handler for when a value of a parameter in the grid has been updated.

    :param frame:
    :param event:
    :return:
    """
    frame.simulation_queue_counter+=1
    if frame.main_frame_menubar.mb_fit_autosim.IsChecked() and not frame.flag_simulating:
        _thread.start_new_thread(simulation_loop, (frame,))

def simulation_loop(frame):
    """ Simulation loop for threading to increase the speed of the interactive simulations
    :param frame:
    :return:
    """
    frame.flag_simulating=True
    while frame.simulation_queue_counter>0:
        do_simulation(frame, from_thread=True)
        time.sleep(0.1)
        frame.simulation_queue_counter=min(1, frame.simulation_queue_counter-1)
    frame.flag_simulating=False

def evaluate(frame, event):
    '''evaluate(frame, event) --> None
    
    Envent handler for only evaluating the Sim function - no recompiling
    '''
    frame.flag_simulating=True
    frame.main_frame_statusbar.SetStatusText('Simulating...', 1)
    # Compile is not necessary when using simualate...
    # frame.model.compile_script()
    try:
        frame.model.simulate(compile=False)
    except modellib.GenericError as e:
        ShowModelErrorDialog(frame, str(e))
        frame.main_frame_statusbar.SetStatusText('Error in simulation', 1)
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, val)
        frame.main_frame_statusbar.SetStatusText('Fatal Error - simulate', 1)
    else:
        _post_sim_plot_event(frame, frame.model, 'Simulation')
        frame.main_frame_statusbar.SetStatusText('Simulation Sucessful', 1)
        frame.plugin_control.OnSimulate(None)
    frame.flag_simulating=False

def simulate(frame, event):
    '''
    simulate(frame, event) --> None
    
    Event handler for simulation.
    '''
    frame.flag_simulating=True
    do_simulation(frame)
    set_possible_parameters_in_grid(frame)
    frame.flag_simulating=False

def do_simulation(frame, from_thread=False):
    if not from_thread: frame.main_frame_statusbar.SetStatusText('Simulating...', 1)
    frame.model.set_script(frame.script_editor.GetText())
    try:
        # when updated from thread and was compiled before, do not compile again
        frame.model.simulate(compile=not (from_thread and frame.model.is_compiled()))
    except modellib.GenericError as e:
        wx.CallAfter(ShowModelErrorDialog, frame, str(e))
        if not from_thread: frame.main_frame_statusbar.SetStatusText('Error in simulation', 1)
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        wx.CallAfter(ShowErrorDialog, frame, val)
        if not from_thread: frame.main_frame_statusbar.SetStatusText('Fatal Error - simulate', 1)
    else:
        wx.CallAfter(_post_sim_plot_event, frame, frame.model, 'Simulation')
        wx.CallAfter(frame.plugin_control.OnSimulate, None)
        if not from_thread: frame.main_frame_statusbar.SetStatusText('Simulation Sucessful', 1)

def set_possible_parameters_in_grid(frame):
    # Now we should find the parameters that we can use to
    # in the grid
    try:
        pardict=frame.model.get_possible_parameters()
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        # ShowErrorDialog(frame, val)
        ShowErrorDialog(frame, val,
                        'simulate - model.get_possible_parameters')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 0)
        return

    try:
        frame.paramter_grid.SetParameterSelections(pardict)
    except Exception as e:
        ShowErrorDialog(frame, str(e),
                        'simulate - parameter_grid.SetParameterSelection')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 0)
        return
    # Set the function for which the parameter can be evaluated with
    frame.paramter_grid.SetEvalFunc(frame.model.eval_in_model)

def start_fit(frame, event):
    '''start_fit(frame, event) --> None
    
    Event handler to start fitting
    '''
    if frame.model.compiled:
        try:
            frame.solver_control.StartFit()
        except modellib.GenericError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Error in fitting', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Fitting starting ...', 1)
    else:
        ShowNotificationDialog(frame, 'The script is not compiled, do a'
                                      ' simulation before you start fitting.')

def stop_fit(frame, event):
    '''stop_fit(frame, event) --> None
    
    Event handler to stop the fitting routine
    '''
    frame.solver_control.StopFit()

def resume_fit(frame, event):
    '''resume_fit(frame, event) --> None
    
    Event handler to resume the fitting routine. No initilization.
    '''
    if frame.model.compiled:
        try:
            frame.solver_control.ResumeFit()
        except modellib.GenericError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Error in fitting', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Fitting starting ...', 1)
    else:
        ShowNotificationDialog(frame, 'The script is not compiled, do a'
                                      ' simulation before you start fitting.')

def calculate_error_bars(frame, evt):
    '''calculate_error_bars(frame, evt) --> None
    
    callback to calculate the error bars on the data.
    '''
    try:
        error_values=frame.solver_control.CalcErrorBars()
    except solvergui.ErrorBarError as e:
        ShowNotificationDialog(frame, str(e))
    except Exception as e:
        ShowErrorDialog(frame, str(e), 'solvergui - CalcErrorBars')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
    else:
        frame.model.parameters.set_error_pars(error_values)
        frame.paramter_grid.SetParameters(frame.model.parameters)
        frame.main_frame_statusbar.SetStatusText('Errorbars calculated', 1)

def error_stats(frame, evt):
    try:
        from .bumps_interface import StatisticalAnalysisDialog
    except Exception as e:
        ShowErrorDialog(frame, str(e), 'solvergui - CalcErrorBars')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
    else:
        dia=StatisticalAnalysisDialog(frame, frame.model)
        dia.ShowModal()

def scan_parameter(frame, row):
    ''' scan_parameter(frame, row) --> None
    
    Scans the parameter in row row [int] from max to min in the number
    of steps given by dialog input.
    '''
    if not frame.model.is_compiled():
        ShowNotificationDialog(frame, 'Please conduct a simulation before'+ \
                               ' scanning a parameter. The script needs to be compiled.')
        return

    dlg=wx.NumberEntryDialog(frame,
                             'Input the number of evaluation points for the scan',
                             'Steps', '', 50, 2, 1000)
    if dlg.ShowModal()==wx.ID_OK:
        frame.main_frame_statusbar.SetStatusText('Scanning parameter', 1)
        try:
            x, y=frame.solver_control.ScanParameter(row, dlg.GetValue())
            fs, pars=frame.model.get_sim_pars()
            bestx=frame.model.parameters.get_data()[row][1]
            besty=frame.model.fom

            frame.plot_fomscan.SetPlottype('scan')
            frame.plot_fomscan.Plot((x, y, bestx, besty,
                                     frame.solver_control.fom_error_bars_level),
                                    frame.model.parameters.get_names()[row],
                                    'FOM')
            frame.sep_plot_notebook.SetSelection(3)
        except Exception as e:
            outp=StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, val)
            frame.main_frame_statusbar.SetStatusText('Fatal Error - scan fom', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Scanning finished', 1)

    dlg.Destroy()

def project_fom_parameter(frame, row):
    '''project_fom_parameter(frame, row) --> None
    
    Plots the project fom given by the row row [int]
    '''
    if not frame.solver_control.IsFitted():
        ShowNotificationDialog(frame, 'Please conduct a fit before'+
                               ' scanning a parameter. The script needs to be compiled and foms have'
                               +' to be collected.')
        return

    frame.main_frame_statusbar.SetStatusText('Trying to project fom', 1)
    try:
        x, y=frame.solver_control.ProjectEvals(row)
        if len(x)==0 or len(y)==0:
            ShowNotificationDialog(frame, 'Please conduct a fit before'+
                                   ' projecting a parameter. The script needs to be compiled and foms have'
                                   +' to be collected.')
            return
        elif frame.model.fom is None or np.isnan(frame.model.fom):
            ShowNotificationDialog(frame, 'The model must be simulated (FOM is not a valid number)')
            return
        fs, pars=frame.model.get_sim_pars()
        bestx=pars[row]
        besty=frame.model.fom
        frame.plot_fomscan.SetPlottype('project')
        frame.plot_fomscan.Plot((x, y, bestx, besty,
                                 frame.solver_control.fom_error_bars_level),
                                frame.model.parameters.get_names()[row],
                                'FOM')
        frame.sep_plot_notebook.SetSelection(3)
    except Exception as e:
        outp=StringIO()
        traceback.print_exc(200, outp)
        val=outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, val)
        frame.main_frame_statusbar.SetStatusText('Fatal Error - project fom', 1)
    else:
        frame.main_frame_statusbar.SetStatusText('Projected fom plotted', 1)

def on_optimizer_settings(frame, event):
    '''on_optimizer_settings(self, event) --> None
    
    Show the settings dialog for the optimizer
    '''
    # try:
    frame.solver_control.ParametersDialog(frame)
    # except Exception, e:
    #    raise e

def on_data_loader_settings(frame, event):
    '''on_data_loader_settings(frame, event) --> None
    
    Show the data_loader settings dialog. Allow the user to change the 
    data loader.
    '''
    frame.data_list.DataLoaderSettingsDialog()

def quit(frame, event):
    '''quit(frame, event) --> None
    
    Quit the program 
    '''
    # Check so the model is saved before quitting
    if not frame.model.saved:
        ans=ShowQuestionDialog(frame, 'If you continue any changes in'
                                      ' your model will not be saved.',
                               'Model not saved')
        if not ans:
            return

    import appdirs
    config_path=appdirs.user_data_dir('GenX3', 'ArturGlavic')+'/'
    hsize, vsize=frame.GetSize()
    frame.config.default_set('gui', 'hsize', hsize)
    frame.config.default_set('gui', 'vsize', vsize)
    frame.config.default_set('gui', 'vsplit', frame.ver_splitter.GetSashPosition())
    frame.config.default_set('gui', 'hsplit', frame.hor_splitter.GetSashPosition())
    frame.config.default_set('gui', 'psplit', frame.plot_splitter.GetSashPosition())
    frame.config.write_default(config_path+'genx.conf')

    # app=frame.parent
    frame.findreplace_dlg.Destroy()
    frame.findreplace_dlg=None
    event.Skip()
    frame.Destroy()
    # app.ExitMainLoop()

def status_text(frame, event):
    '''status_text(frame, event) --> None
    Print a status text in the window. event should have a string
    member text. This will display the message in the status bar.
    '''
    frame.main_frame_statusbar.SetStatusText(event.text, 1)

def fom_value(frame, event):
    '''fom_value(frame, event) --> None
    
    Callback to update the fom_value displayed by the gui
    '''
    fom_value=event.model.fom
    fom_name=event.model.fom_func.__name__
    if fom_value:
        frame.main_frame_fom_text.SetLabel('        FOM %s: %.4e'%(fom_name, fom_value))
    else:
        frame.main_frame_fom_text.SetLabel('        FOM %s: None'%fom_name)

def point_pick(frame, event):
    '''point_pick(frame, event) --> None
    Callback for the picking of a data point in a plotting window.
    This will display the message in the status bar.
    '''
    frame.main_frame_statusbar.SetStatusText(event.text, 2)

def on_zoom_check(frame, event):
    '''on_zoom_toolbar(event) --> none
    
    Takes care of clicks on the toolbar zoom button and the menu item zoom.
    '''
    if event.GetId()==10009:
        zoom_state=frame.main_frame_toolbar.GetToolState(10009)
        frame.main_frame_menubar.mb_view_zoom.Check(zoom_state)
    else:
        zoom_state=frame.main_frame_menubar.mb_view_zoom.IsChecked()
        frame.main_frame_toolbar.ToggleTool(10009, zoom_state)

    # Synchronize all plots with zoom state
    pages=get_pages(frame)
    for page in pages:
        page.SetZoom(zoom_state)

def zoomall(frame, event):
    '''zoomall(self, event) --> None
    
    Zoom out and show all data points
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        tmp=pages[sel].GetAutoScale()
        pages[sel].SetAutoScale(True)
        pages[sel].AutoScale()
        pages[sel].SetAutoScale(tmp)
        pages[sel].AutoScale()

def set_yscale(frame, type):
    '''set_yscale(frame, type) --> None
    
    Set the y-scale of the current plot. type should be linear or log, strings. 
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].SetYScale(type)

def set_xscale(frame, type):
    '''set_xscale(frame, type) --> None

    Set the x-scale of the current plot. type should be linear or log, strings.
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].SetXScale(type)

def on_grid_slider_check(frame, event):
    """Change the state of the grid value input, either as slider or as a number.
    :param frame:
    :param event:
    :return:
    """
    frame.paramter_grid.SetValueEditorSlider(frame.main_frame_menubar.mb_view_grid_slider.IsChecked())
    # print frame.paramter_grid.get_toggle_slider_tool_state()
    frame.paramter_grid.toggle_slider_tool(frame.main_frame_menubar.mb_view_grid_slider.IsChecked())
    frame.paramter_grid.Refresh()

def on_autoscale(frame, event):
    '''on_autoscale(frame, event) --> None
    
    Toggles the autoscale of the current plot.
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].SetAutoScale(not pages[sel].GetAutoScale())

def plot_settings_changed(frame, event):
    '''zoom_changed(frame, event) --> None
    
    Callback for the settings change event for the current plot
     - change the toggle for the zoom icon and change the menu items.
    '''
    frame.main_frame_toolbar.ToggleTool(10009, event.zoomstate)
    frame.main_frame_menubar.mb_view_zoom.Check(event.zoomstate)
    if event.yscale=='log':
        frame.main_frame_menubar.mb_view_yscale_log.Check(True)
    elif event.yscale=='linear':
        frame.main_frame_menubar.mb_view_yscale_lin.Check(True)
    if event.xscale=='log':
        frame.main_frame_menubar.mb_view_xscale_log.Check(True)
    elif event.xscale=='linear':
        frame.main_frame_menubar.mb_view_xscale_lin.Check(True)
    frame.main_frame_menubar.mb_view_autoscale.Check(event.autoscale)

def plot_page_changed(frame, event):
    '''plot_page_changed(frame, event) --> None
        
    Callback for page change in plot notebook. Changes the state of
    the zoom toggle button.
    '''
    sel=event.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        zoom_state=pages[sel].GetZoom()
        # Set the zoom button to the correct value
        frame.main_frame_toolbar.ToggleTool(10009, zoom_state)
        frame.main_frame_menubar.mb_view_zoom.Check(zoom_state)

        yscale=pages[sel].GetYScale()
        if yscale=='log':
            frame.main_frame_menubar.mb_view_yscale_log.Check(True)
        elif yscale=='linear':
            frame.main_frame_menubar.mb_view_yscale_lin.Check(True)
        xscale=pages[sel].GetXScale()
        if xscale=='log':
            frame.main_frame_menubar.mb_view_yscale_log.Check(True)
        elif xscale=='linear':
            frame.main_frame_menubar.mb_view_yscale_lin.Check(True)

def print_plot(frame, event):
    '''print_plot(frame, event) --> None
    
    prints the current plot in the plot notebook.
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].Print()

def print_preview_plot(frame, event):
    '''print_preview_plot(frame, event) --> None
    
    prints a preview of the current plot int the plot notebook.
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].PrintPreview()

def print_parameter_grid(frame, event):
    ''' print_parameter_grid(frame, event) --> None
    
    Prints the table of parameters that have been fitted.
    '''
    frame.paramter_grid.Print()
    # genxprint.TablePanel(frame)

def print_preview_parameter_grid(frame, event):
    ''' print_parameter_grid(frame, event) --> None
    
    Prints the table of parameters that have been fitted.
    '''
    frame.paramter_grid.PrintPreview()

def copy_graph(frame, event):
    '''copy_graph(self, event) --> None
    
    Callback that copies the current graph in the plot notebook to
    the clipboard.
    '''
    sel=frame.sep_plot_notebook.GetSelection()
    pages=get_pages(frame)
    if sel<len(pages):
        pages[sel].CopyToClipboard()

def copy_table(frame, event):
    '''copy_table(frame, event) --> None
    
    Copies the table as ascii text to the clipboard
    '''
    ascii_table=frame.paramter_grid.table.pars.get_ascii_output()
    text_table=wx.TextDataObject(ascii_table)
    if wx.TheClipboard.Open():
        wx.TheClipboard.SetData(text_table)
        wx.TheClipboard.Close()

def copy_sim(frame, event):
    '''copy_sim(frame, event) --> None
    
    Copies the simulation and the data to the clipboard. Note that this
    copies ALL data.
    '''
    text_string=frame.model.get_data_as_asciitable()
    text=wx.TextDataObject(text_string)
    if wx.TheClipboard.Open():
        wx.TheClipboard.SetData(text)
        wx.TheClipboard.Close()

def on_findreplace(frame, event):
    '''Show the find and replace dialog box.
    '''
    frame.findreplace_dlg.Show(True)

def on_find_event(frame, event):
    '''callback for find events - coupled to the script
    '''
    evtype=event.GetEventType()

    def find():
        find_str=event.GetFindString()
        ##print frame.findreplace_data.GetFlags()
        flags=event.GetFlags()
        if flags & 1:
            ##print "Searching down"
            pos=frame.script_editor.SearchNext(flags, find_str)
        else:
            ##print "Searching up"
            pos=frame.script_editor.SearchPrev(flags, find_str)
        if pos==-1:
            frame.main_frame_statusbar.SetStatusText(
                'Could not find text %s'%find_str, 1)
        return pos

    def replace():
        replace_str=event.GetReplaceString()
        frame.script_editor.ReplaceSelection(replace_str)

    # Deal with the different cases
    if evtype==wx.wxEVT_COMMAND_FIND:
        frame.script_editor.SearchAnchor()
        find()

    elif evtype==wx.wxEVT_COMMAND_FIND_NEXT:
        pnew=frame.script_editor.GetSelectionEnd()
        ##print pnew
        frame.script_editor.GotoPos(pnew)
        frame.script_editor.SetAnchor(pnew)
        frame.script_editor.SearchAnchor()
        ##print 'Finding next'
        find()

    elif evtype==wx.wxEVT_COMMAND_FIND_REPLACE:
        # print 'find and replace'
        # If we do not have found text already
        # or if we have marked other text by mistake...
        if frame.script_editor.GetSelectedText()!= \
                event.GetFindString():
            find()
        # We already have found and marked text that we should
        # replace
        else:
            frame.script_editor.ReplaceSelection(
                event.GetReplaceString())
            # Find a new text to replace
            find()
    elif evtype==wx.wxEVT_COMMAND_FIND_REPLACE_ALL:
        # print 'find and replace all'
        if frame.script_editor.GetSelectedText()!= \
                event.GetFindString():
            pos=find()
        i=0
        while pos!=-1:
            frame.script_editor.ReplaceSelection(
                event.GetReplaceString())
            i+=1
            pos=find()
        frame.main_frame_statusbar.SetStatusText(
            'Replaces %d occurancies of  %s'%(i,
                                              event.GetFindString()), 1)

    ##else:
    ##    ShowErrorDialog(frame, 'Faulty event supplied in find and'\
    ##                    ' repalce functionallity', 'on_find_event')
    # This will scroll the editor to the right position so we can see
    # the text
    frame.script_editor.EnsureCaretVisible()

def change_data_grid_view(frame, event):
    '''change_data_grid_view(frame, event) --> None
    
    change the data displayed in the grid...
    '''
    # print event.GetSelection()
    dataset=frame.model.data[event.GetSelection()]
    rows=frame.data_grid.GetNumberRows()
    new_rows=max(len(dataset.x), len(dataset.y),
                 len(dataset.x_raw), len(dataset.y_raw))
    frame.data_grid.DeleteRows(numRows=rows)
    frame.data_grid.AppendRows(new_rows)
    [[frame.data_grid.SetCellValue(row, col, '-') for col in range(6)] \
     for row in range(new_rows)]
    [frame.data_grid.SetCellValue(row, 0, '%.3e'%dataset.x_raw[row]) \
     for row in range(len(dataset.x_raw))]
    [frame.data_grid.SetCellValue(row, 1, '%.3e'%dataset.y_raw[row]) \
     for row in range(len(dataset.y_raw))]
    [frame.data_grid.SetCellValue(row, 2, '%.3e'%dataset.error_raw[row]) \
     for row in range(len(dataset.error_raw))]
    [frame.data_grid.SetCellValue(row, 3, '%.3e'%dataset.x[row]) \
     for row in range(len(dataset.x))]
    [frame.data_grid.SetCellValue(row, 4, '%.3e'%dataset.y[row]) \
     for row in range(len(dataset.y))]
    [frame.data_grid.SetCellValue(row, 5, '%.3e'%dataset.error[row]) \
     for row in range(len(dataset.error))]

def update_data_grid_choice(frame, event):
    '''update_data_grid_choice(frame, event) --> None
    
    Updates the choices of the grids to display from the data.
    '''
    data=event.GetData()
    names=[data_set.name for data_set in data]
    # frame.data_grid_choice.SetItems(names)
    frame.data_grid_choice.Clear()
    frame.data_grid_choice.AppendItems(names)
    event.Skip()

def update_data(frame, event):
    '''update_data(frame, event) --> None
    
    callback for updating data, right now in the plugins
    '''
    frame.plugin_control.OnDataChanged(event)

def activate_cuda(frame, event):
    dlg=wx.ProgressDialog(parent=frame,
                          maximum=3,
                          message="Compiling CUDA GPU computing functions with Numba",
                          title='Activating CUDA...')
    dlg.Show()

    dlg.Update(1)
    from genx.models.lib import paratt_cuda
    dlg.Update(2)
    from genx.models.lib import neutron_cuda
    dlg.Update(3)
    from models.lib import paratt, neutron_refl
    paratt.Refl=paratt_cuda.Refl
    paratt.ReflQ=paratt_cuda.ReflQ
    paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
    neutron_refl.Refl=neutron_cuda.Refl
    from genx.models.lib import paratt, neutron_refl
    paratt.Refl=paratt_cuda.Refl
    paratt.ReflQ=paratt_cuda.ReflQ
    paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
    neutron_refl.Refl=neutron_cuda.Refl

    dlg.Destroy()

def deactivate_cuda(frame, event):
    from genx.models.lib import paratt_numba, neutron_numba
    from models.lib import paratt, neutron_refl
    paratt.Refl=paratt_numba.Refl
    paratt.ReflQ=paratt_numba.ReflQ
    paratt.Refl_nvary2=paratt_numba.Refl_nvary2
    neutron_refl.Refl=neutron_numba.Refl
    from genx.models.lib import paratt, neutron_refl
    paratt.Refl=paratt_numba.Refl
    paratt.ReflQ=paratt_numba.ReflQ
    paratt.Refl_nvary2=paratt_numba.Refl_nvary2
    neutron_refl.Refl=neutron_numba.Refl

def fom_help(frame, event):
    '''Show a help dialog for information about the different fom.
    '''
    dlg=help.PluginHelpDialog(frame, 'fom_funcs', title="FOM functions help")
    dlg.Show()

def models_help(frame, event):
    '''models_help(frame, event) --> None
    
    Show a help dialog for information about the different models.
    '''
    dlg=help.PluginHelpDialog(frame, 'models', title="Models help")
    if frame.model.is_compiled():
        current_model=frame.model.script_module.model.__name__.rsplit('.', 1)[1]
        if current_model in dlg.choice.GetStrings():
            dlg.choice.SetStringSelection(current_model)
            dlg.on_choice(None)
    dlg.Show()

def plugins_help(frame, event):
    '''plugins_help(frame, event) --> None
    
    Show a help dialog for information about the different plugins.
    '''
    dlg=help.PluginHelpDialog(frame, 'plugins.add_ons', title="Plugins help")
    dlg.Show()

def data_loaders_help(frame, event):
    '''data_loaders_help(frame, event) --> None
    
    Show a help dialog for information about the different data_loaders.
    '''
    dlg=help.PluginHelpDialog(frame, 'plugins.data_loaders', title="Data loaders help")
    dlg.Show()

def show_manual(frame, event):
    '''show_manual(frame, event) --> None
    
    Callback to show the manual
    '''
    # ShowNotificationDialog(frame, 'There is no manual yet!')
    webbrowser.open_new(manual_url)

def show_homepage(frame, event):
    '''show_homepage(frame, event) --> None
    
    Callback to show the homepage
    '''
    webbrowser.open_new(homepage_url)

def show_about_box(frame, event):
    '''show_about_box(frame, event) --> None
    
    Show an about box about GenX with some info...
    '''
    import numpy, scipy, matplotlib, platform
    useful=''
    try:
        import numba
        useful+='Numba: %s, '%numba.__version__
    except:
        pass
    try:
        import vtk
        useful+='VTK: %s, '%vtk.vtkVersion.GetVTKVersion()
    except:
        pass
    try:
        import dumps
        useful+='Dumps: %s, '%dumps.__version__
    except:
        pass

    info=wx.adv.AboutDialogInfo()
    info.Name="GenX"
    info.Version=__version__
    info.Copyright="(C) 2008 Matts Bjorck; 2020 Artur Glavic"
    info.Description=wordwrap(
        "GenX is a multipurpose refinement program using the differential "
        "evolution algorithm. It is developed  mainly for refining x-ray reflectivity "
        "and neutron reflectivity data."

        "\n\nThe versions of the mandatory libraries are:\n"
        "Python: %s, wxPython: %s, Numpy: %s, Scipy: %s, Matplotlib: %s"
        "\n\nThe non-mandatory but useful packages:\n%s"
        ""%(platform.python_version(), wx.__version__,
            numpy.__version__, scipy.__version__,
            matplotlib.__version__, useful),
        500, wx.ClientDC(frame))
    info.WebSite=("http:////genx.sourceforge.net", "GenX homepage")
    # No developers yet
    # info.Developers = []
    # head, tail = os.path.split(__file__)
    # license_text = file(head + '/LICENSE.txt','r').read()
    # license_text = file(_path + 'LICENSE.txt','r').read()
    # info.License = license_text#wordwrap(license_text, 500, wx.ClientDC(self))
    info.Licence=wordwrap('This program is free software: you can redistribute it and/or modify '
                          'it under the terms of the GNU General Public License as published by '
                          'the Free Software Foundation, either version 3 of the License, or '
                          '(at your option) any later version. '
                          '\n\nThis program is distributed in the hope that it will be useful, '
                          'but WITHOUT ANY WARRANTY; without even the implied warranty of '
                          'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the '
                          'GNU General Public License for more details. '
                          '\n\nYou should have received a copy of the GNU General Public License '
                          'along with this program.  If not, see <http://www.gnu.org/licenses/>. '
                          , 400, wx.ClientDC(frame))

    wx.adv.AboutBox(info)

# =============================================================================
# Custom events needed for updating and message parsing between the different
# modules.

class GenericModelEvent(wx.CommandEvent):
    '''
    Event class for a new model - for updating
    of the paramters, plots and script.
    '''

    def __init__(self, evt_type, id, model):
        wx.CommandEvent.__init__(self, evt_type, id)
        self.model=model
        self.description=''

    def GetModel(self):
        return self.model

    def SetModel(self, model):
        self.model=model

    def SetDescription(self, desc):
        '''
        Set a string that describes the event that has occurred
        '''
        self.description=desc

# Generating an event type:
myEVT_NEW_MODEL=wx.NewEventType()
# Creating an event binder object
EVT_NEW_MODEL=wx.PyEventBinder(myEVT_NEW_MODEL)

def _post_new_model_event(parent, model, desc=''):
    # Send an event that a new data set has been loaded
    evt=GenericModelEvent(myEVT_NEW_MODEL, parent.GetId(), model)
    evt.SetDescription(desc)
    # Process the event!
    parent.GetEventHandler().ProcessEvent(evt)

# Generating an event type:
myEVT_SIM_PLOT=wx.NewEventType()
# Creating an event binder object
EVT_SIM_PLOT=wx.PyEventBinder(myEVT_SIM_PLOT)

def _post_sim_plot_event(parent, model, desc=''):
    # Send an event that a new data set ahs been loaded
    evt=GenericModelEvent(myEVT_SIM_PLOT, parent.GetId(), model)
    evt.SetDescription(desc)
    # Process the event!
    parent.GetEventHandler().ProcessEvent(evt)

# ==============================================================================
## Functions for showing error dialogs

def ShowQuestionDialog(frame, message, title='Question?'):
    dlg=wx.MessageDialog(frame, message,
                         title,
                         wx.OK | wx.CANCEL | wx.OK_DEFAULT | wx.ICON_QUESTION
                         )
    result=dlg.ShowModal()==wx.ID_OK
    dlg.Destroy()
    return result

def ShowModelErrorDialog(frame, message):
    dlg=wx.MessageDialog(frame, message,
                         'Warning',
                         wx.OK | wx.ICON_WARNING
                         )
    dlg.ShowModal()
    dlg.Destroy()

def ShowNotificationDialog(frame, message):
    dlg=wx.MessageDialog(frame, message,
                         'Information',
                         wx.OK | wx.ICON_INFORMATION
                         )
    dlg.ShowModal()
    dlg.Destroy()

def ShowErrorDialog(frame, message, position=''):
    if position!='':
        dlg=wx.MessageDialog(frame, message+'\n'+'Position: '+position,
                             'FATAL ERROR',
                             wx.OK | wx.ICON_ERROR
                             )
    else:
        dlg=wx.MessageDialog(frame, message,
                             'FATAL ERROR',
                             wx.OK | wx.ICON_ERROR
                             )
    dlg.ShowModal()
    dlg.Destroy()
# ==============================================================================
